#!/usr/bin/python
# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author:
__author__ =  'matt.clark@eucalyptus.com'
'''
Test case class to test points of network security groups
See individual test descriptions for test objectives.
test1:
    Definition:
        Create test instances within each zone within security group1. This security group is authorized for
        ssh access from 0.0.0.0/0.
        This test attempts the following:
            -To run an instance in each zone and confirm it reaches 'running' state.
            -Confirm the instance is ping-able from the cc within a given timeout
            -Establish and verify an ssh session directly from the local machine running this test.
            -Place ssh key on instance for later use
            -Add instance to global 'group1_instances'

test2:
    Definition:
        This test attempts to create an instance in each within security group2 which should not
        be authorized for any remote access (outside of the CC).
        The test attempts the following:
            -To run an instance in each zone and confirm it reaches 'running' state.
            -Confirm the instance is ping-able from the cc within a given timeout
            -Establish and verify an ssh session using the cc as a proxy.
            -Place ssh key on instance for later use
            -Add instance to global 'group2_instances'

test3:
    Definition:
        This test attempts to set up security group rules between group1 and group2 to authorize group2 access
        from group1. If use_cidr is True security groups will be setup using cidr notication ip/mask for each instance in
        group1, otherwise the entire source group 1 will authorized.
        the group will be
        Test attempts to:
            -Authorize security groups for inter group private ip access.
            -Iterate through each zone and attempt to ssh from an instance in group1 to an instance in group2 over their
                private ips.

test4:
    Definition:
        Test attempts to verify that the local machine cannot ssh to the instances within group2 which is not authorized
        for ssh access from this source.

test5 (Multi-zone/cluster env):
    Definition:
        This test attempts to check connectivity for instances in the same security group, but in different zones.
        Note: This test requires the CC have tunnelling enabled, or the CCs in each zone be on same
        layer 2 network segment.
        Test attempts to:
            -Iterate through each zone and attempt to ssh from an instance in group1 to an instance in a separate zone
             but same security group1 over their private ips.

test 6 (Multi-zone/cluster env):
    Definition:
        This test attempts to set up security group rules between group1 and group2 to authorize group2 access
        from group1 across different zones.
        If no_cidr is True security groups will be setup using cidr notication ip/mask for each instance in
        group1, otherwise the entire source group 1 will authorized.
        the group will be
        Note: This test requires the CC have tunnelling enabled, or the CCs in each zone be on same
        layer 2 network segment.
        Test attempts to:
            -Authorize security groups for inter group private ip access.
            -Iterate through each zone and attempt to ssh from an instance in group1 to an instance in group2 over their
             private ips.


'''

#todo: Make use of CC optional so test can be run with only creds and non-admin user.
# CC only provides additional point of debug so can be removed from test for non-euca testing
#todo: Allow test to run with an admin and non-admin account, so debug can be provided through admin and test can
# be run under non-admin if desired.


from eucaops import Eucaops
from eutester.eutestcase import EutesterTestCase
from eutester.eutestcase import SkipTestException
from eutester.euinstance import EuInstance
from eutester.sshconnection import SshConnection
import time
import os
import sys
import copy


class Net_Tests(EutesterTestCase):

    def __init__(self,  tester=None, **kwargs):
        self.setuptestcase()
        self.setup_parser()
        self.parser.add_argument("--no_cidr",
                                 action='store_false',
                                 help="Boolean to authorize sec group with cidr notation or by group ",
                                 default=True)
        self.parser.add_argument("--freeze_on_fail",
                                 action='store_true',
                                 help="Boolean flag to avoid cleaning test resources upon failure, default: True ",
                                 default=False)
        '''
        self.parser.add_argument('--user',
                                 default='Admin',
                                 help='User to run this test as')
        self.parser.add_argument("--account",
                                 help="Account to run this as",
                                 default=None)
        '''
        self.tester = tester
        self.get_args()
        # Allow __init__ to get args from __init__'s kwargs or through command line parser...
        for kw in kwargs:
            print 'Setting kwarg:'+str(kw)+" to "+str(kwargs[kw])
            self.set_arg(kw ,kwargs[kw])
        self.show_args()
        ### Create the Eucaops object, by default this will be Eucalyptus/Admin and have ssh access to components
        if not tester and not self.args.config:
            print "Need eutester config file to execute this test. As well as system ssh credentials (key, password, etc)"
            self.parser.print_help()
            sys.exit(1)
        # Setup basic eutester object
        if not self.tester:
            try:
                self.debug('Creating Eucaops tester object from args provided...')
                self.tester = self.do_with_args(Eucaops)
            except Exception, e:
                raise Exception('Couldnt create Eucaops tester object, make sure credpath, '
                                'or config_file and password was provided, err:' + str(e))
                #replace default eutester debugger with eutestcase's for more verbosity...
            self.tester.debug = lambda msg: self.debug(msg, traceback=2, linebyline=False)
        assert isinstance(self.tester, Eucaops)
        self.cc_last_checked = time.time()

        ### Create local zone list to run tests in
        if self.args.zone:
            self.zones = [str(self.args.zone)]
        else:
            self.zones = self.tester.get_zones()
        if not self.zones:
            raise Exception('No zones found to run this test?')
        self.debug('Running test against zones:' + ",".join(self.zones))

        ### Add and authorize securtiy groups
        self.debug("Creating group1..")
        self.group1 = self.tester.add_group(str(self.name) + "_group1_" + str(time.time()))
        self.debug("Authorize ssh for group1 from '0.0.0.0/0'")
        self.tester.authorize_group(self.group1, port=22, protocol='tcp', cidr_ip='0.0.0.0/0')
        #self.tester.authorize_group(self.group1, protocol='icmp',port='-1')

        self.debug("Creating group2, will authorize later from rules within test methods..")
        self.group2 = self.tester.add_group(str(self.name) + "_group2_" + str(time.time()))
        self.group1_instances = []
        self.group2_instances = []



        ### Generate a keypair for the instances
        try:
            keys = self.tester.get_all_current_local_keys()
            if keys:
                self.keypair = keys[0]
            else:
                self.keypair = self.tester.add_keypair(str(self.name) + "_key_" + str(time.time()))
        except Exception, ke:
            raise Exception("Failed to find/create a keypair, error:" + str(ke))

        ### Get an image to work with
        if self.args.emi:
            self.image = self.tester.get_emi(emi=str(self.args.emi))
        else:
            self.image = self.tester.get_emi(root_device_type="instance-store")
        if not self.image:
            raise Exception('couldnt find instance store image')




    ######################################################
    #   Test Utility Methods
    ######################################################

    def authorize_group_for_instance_list(self, group, instances):
        for instance in instances:
            assert isinstance(instance, EuInstance)
            self.tester.authorize_group(group, cidr_ip=instance.private_ip_address + "/32")

    def clean_method(self):
        if self.args.freeze_on_fail:
            self.status('freeze_on_fail arg set, not cleaning test resources')
        else:
            self.tester.cleanup_artifacts()

    def get_proxy_machine(self, instance):
        if self.tester.config["network"].lower() == "edge":
            proxy_machine = self.get_active_nc_for_instance(instance)
        else:
            proxy_machine = self.get_active_cc_for_instance(instance)
        self.debug("Instance is running on: " + proxy_machine.hostname)
        return proxy_machine

    def create_ssh_connection_to_instance(self, instance, retry=10):
        proxy_machine = self.get_proxy_machine(instance)
        ssh = None
        attempts = 0
        elapsed = 0
        next_retry_time = 10
        start = time.time()
        proxy_keypath=proxy_machine.machine.ssh.keypath or None
        while not ssh and attempts < retry:
            attempts += 1
            elapsed = int(time.time()-start)
            self.debug('Attempting to ssh to instances private ip:' + str(instance.private_ip_address) +
                       'through the cc ip:' + str(proxy_machine.hostname) + ', attempts:' +str(attempts) + "/" + str(retry) +
                       ", elapsed:" + str(elapsed))
            try:
                ssh = SshConnection(host=instance.private_ip_address,
                                keypath=instance.keypath,
                                proxy=proxy_machine.hostname,
                                proxy_username=proxy_machine.machine.ssh.username,
                                proxy_password=proxy_machine.machine.ssh.password,
                                proxy_keypath=proxy_keypath)
            except Exception, ce:
                tb = self.tester.get_traceback()
                if attempts >= retry:
                    self.debug("\n" + tb,linebyline=False)
                self.debug('Failed to connect error:' + str(ce))
            if attempts < retry:
                    time.sleep(next_retry_time)

        if not ssh:
            raise Exception('Could not ssh to instances private ip:' + str(instance.private_ip_address) +
                            ' through the cc ip:' + str(proxy_machine.hostname) + ', attempts:' +str(attempts) + "/" + str(retry) +
                            ", elapsed:" + str(elapsed))

        return ssh

    def get_active_cc_for_instance(self,instance,refresh_active_cc=30):
        elapsed = time.time()-self.cc_last_checked
        self.cc_last_checked = time.time()
        if elapsed > refresh_active_cc:
            use_cached_list = False
        else:
            use_cached_list = True
        cc = self.tester.service_manager.get_all_cluster_controllers(partition=instance.placement,
                                                                     use_cached_list= use_cached_list,
                                                                     state='ENABLED')[0]
        return cc

    def get_active_nc_for_instance(self,instance):
        nc = self.tester.service_manager.get_all_node_controllers(instance_id=instance.id, use_cached_list=False).pop()
        return nc

    def ping_instance_private_ip_from_active_cc(self, instance):
        assert isinstance(instance, EuInstance)
        proxy_machine = self.get_proxy_machine(instance)
        try:
            proxy_machine.machine.ping_check(instance.private_ip_address)
            return True
        except:pass
        return False




    ################################################################
    #   Test Methods
    ################################################################


    def test1_create_instance_in_zones_for_security_group1(self, ping_timeout=180):
        '''
        Definition:
        Create test instances within each zone within security group1. This security group is authorized for
        ssh access from 0.0.0.0/0.
        This test attempts the following:
            -To run an instance in each zone and confirm it reaches 'running' state.
            -Confirm the instance is ping-able from the cc within a given timeout
            -Establish and verify an ssh session directly from the local machine running this test.
            -Place ssh key on instance for later use
            -Add instance to global 'group1_instances'
        '''
        for zone in self.zones:
            #Create an instance, monitor it's state but disable the auto network/connect checks till afterward
            instance = self.tester.run_image(image=self.image,
                                             keypair=self.keypair,
                                             group=self.group1,
                                             zone=zone,
                                             auto_connect=False,
                                             monitor_to_running=False)[0]
            self.group1_instances.append(instance)
        self.tester.monitor_euinstances_to_running(self.group1_instances)
        #Now run the network portion.
        for instance in self.group1_instances:
            self.status('Checking connectivity to:' + str(instance.id) + ":" + str(instance.private_ip_address)+
                        ", zone:" + str(instance.placement) )
            assert isinstance(instance, EuInstance)
            self.debug('Attempting to ping instances private ip from cc...')
            self.tester.wait_for_result( self.ping_instance_private_ip_from_active_cc,
                                         result=True,
                                         timeout=ping_timeout,
                                         instance=instance)
            self.debug('Attempting to ssh to instance from local test machine...')
            self.debug('Check some debug information re this data connection in this security group first...')
            self.tester.does_instance_sec_group_allow(instance=instance,
                                                      src_addr=None,
                                                      protocol='tcp',
                                                      port=22)
            instance.connect_to_instance(timeout=90)
            self.status('SSH connection to instance:' + str(instance.id) +
                        ' successful to public ip:' + str(instance.ip_address) +
                        ', zone:' + str(instance.placement))
            instance.sys('uname -a', code=0)
            instance.ssh.sftp_put(instance.keypath, os.path.basename(instance.keypath))
            instance.sys('chmod 0600 ' + os.path.basename(instance.keypath), code=0 )



    def test2_create_instance_in_zones_for_security_group2(self, ping_timeout=180):
        '''
        Definition:
        This test attempts to create an instance in each zone within security group2 which should not
        be authorized for any remote access (outside of the CC).
        The test attempts the following:
            -To run an instance in each zone and confirm it reaches 'running' state.
            -Confirm the instance is ping-able from the cc within a given timeout
            -Establish and verify an ssh session using the cc as a proxy.
            -Place ssh key on instance for later use
            -Add instance to global 'group2_instances'
        '''
        for zone in self.zones:
            instance = self.tester.run_image(image=self.image,
                                             keypair=self.keypair,
                                             group=self.group2,
                                             zone=zone,
                                             auto_connect=False,
                                             monitor_to_running=False)[0]
            self.group2_instances.append(instance)
        self.tester.monitor_euinstances_to_running(self.group2_instances)
        for instance in self.group2_instances:
            self.status('Checking connectivity to:' + str(instance.id) + ":" + str(instance.private_ip_address)+
                        ", zone:" + str(instance.placement) )
            assert isinstance(instance, EuInstance)
            self.tester.wait_for_result( self.ping_instance_private_ip_from_active_cc,
                                         result=True,
                                         timeout=ping_timeout,
                                         instance=instance)
            self.status('Make sure ssh is working through CC path before trying between instances...')
            instance.cc_ssh = self.create_ssh_connection_to_instance(instance)
            self.status('SSH connection to instance:' + str(instance.id) +
                        ' successful to private ip:' + str(instance.private_ip_address) +
                        ', zone:' + str(instance.placement))
            instance.cc_ssh.sys('uname -a', code=0)
            self.status('Uploading keypair to instance in group2...')
            instance.cc_ssh.sftp_put(instance.keypath, os.path.basename(instance.keypath))
            instance.cc_ssh.sys('chmod 0600 ' + os.path.basename(instance.keypath), code=0 )
            self.status('Done with create instance security group2:' + str(instance.id))


    def test3_test_ssh_between_instances_in_diff_sec_groups_same_zone(self, no_cidr=None):
        '''
        Definition:
        This test attempts to set up security group rules between group1 and group2 to authorize group2 access
        from group1. If no_cidr is True security groups will be setup using cidr notation ip/mask for each instance in
        group1, otherwise the entire source group 1 will be authorized.

        Test attempts to:
            -Authorize security groups for inter group private ip access.
            -Iterate through each zone and attempt to ssh from an instance in group1 to an instance in group2 over their
                private ips.
        '''

        if no_cidr is None:
            no_cidr = self.args.no_cidr
        if no_cidr:
            self.authorize_group_for_instance_list(self.group2, self.group1_instances)
        else:
            self.tester.authorize_group(self.group2, cidr_ip=None, port=None, src_security_group_name=self.group1.name )

        for zone in self.zones:
            instance1 = None
            instance2 = None
            for instance in self.group1_instances:
                if instance.placement == zone:
                    assert isinstance(instance, EuInstance)
                    instance1 = instance
                    break
            if not instance1:
                raise Exception('Could not find instance in group1 for zone:' + str(zone))

            for instance in self.group2_instances:
                if instance.placement == zone:
                    assert isinstance(instance, EuInstance)
                    instance2 = instance
                    break
            if not instance2:
                raise Exception('Could not find instance in group2 for zone:' + str(zone))
        self.debug('Attempting to run ssh command "uname -a" between instances across security groups:\n'
                   + str(instance1.id) + '/sec grps(' + str(instance1.security_groups)+") --> "
                   + str(instance2.id) + '/sec grps(' + str(instance2.security_groups)+")\n"
                   + "Current test run in zone: " + str(zone), linebyline=False )
        self.debug('Check some debug information re this data connection in this security group first...')
        self.tester.does_instance_sec_group_allow(instance=instance2,
                                                  src_addr=instance1.private_ip_address,
                                                  protocol='tcp',
                                                  port=22)
        self.debug('Now Running the ssh command...')
        instance1.sys("ssh -o StrictHostKeyChecking=no -i "
                      + str(os.path.basename(instance1.keypath))
                      + " root@" + str(instance2.private_ip_address)
                      + " 'uname -a'", code=0)
        self.debug('Ssh between instances passed')

    def test4_attempt_unauthorized_ssh_from_test_machine_to_group2(self):
        '''
        Description:
        Test attempts to verify that the local machine cannot ssh to the instances within group2 which is not authorized
        for ssh access from this source.
        '''
        for instance in self.group2_instances:
            assert isinstance(instance, EuInstance)
            #Provide some debug information re this data connection in this security group
            self.tester.does_instance_sec_group_allow(instance=instance, src_addr=None, protocol='tcp',port=22)
            try:
                instance.reset_ssh_connection(timeout=5)
                raise Exception('Was able to connect to instance: ' + str(instance.id) + ' in security group:'
                                + str(self.group2.name))
            except:
                self.debug('Success: Was not able to ssh from the local machine to instance in unauthorized sec group')

    def test5_test_ssh_between_instances_in_same_sec_groups_different_zone(self):
        '''
        Definition:
        This test attempts to check connectivity for instances in the same security group, but in different zones.
        Note: This test requires the CC have tunnelling enabled, or the CCs in each zone be on same
        layer 2 network segment.

        Test attempts to:
            -Iterate through each zone and attempt to ssh from an instance in group1 to an instance in a separate zone
             but same security group1 over their private ips.
        '''
        zones = []
        if len(self.zones) < 2:
            raise SkipTestException('Skipping test5, only a single zone found or provided')

        class TestZone():
            def __init__(self, zone):
                self.zone = zone
                self.test_instance_group1 = None
                self.test_instance_group2 = None

        for zone in self.zones:
            zones.append(TestZone(zone))
            #Grab a single instance from each zone within security group1
        for zone in zones:
            instance = None
            for instance in self.group1_instances:
                if instance.placement == zone.zone:
                    assert isinstance(instance, EuInstance)
                    zone.test_instance_group1 = instance
                    break
                instance = None
            if not zone.test_instance_group1:
                raise Exception('Could not find an instance in group1 for zone:' + str(zone.zone))

        self.debug('Iterating through zones, attempting ssh between zones within same security group...')
        for zone in zones:
            instance1 = zone.test_instance_group1
            for zone2 in zones:
                if zone.zone != zone2.zone:
                    instance2 = zone2.test_instance_group1
                    if not instance1 or not instance2:
                        raise Exception('Security group: ' + str(self.group1.name) + ", missing instances in a Zone:"
                                        + str(zone.zone) + " = instance:" + str(instance1) +
                                        ", Zone:" + str(zone2.zone) + " = instance:" + str(instance2))
                    self.debug('Attempting to run ssh command "uname -a" between instances across zones and security groups:\n'
                               + str(instance1.id) + '/sec grps(' + str(instance1.security_groups)+") --> "
                               + str(instance2.id) + '/sec grps(' + str(instance2.security_groups)+")\n"
                               + "Current test run in zones: " + str(instance1.placement) + "-->" + str(instance2.placement),
                               linebyline=False )
                    self.debug('Check some debug information re this data connection in this security group first...')
                    self.tester.does_instance_sec_group_allow(instance=instance2,
                                                              src_addr=instance1.private_ip_address,
                                                              protocol='tcp',
                                                              port=22)
                    self.debug('Now Running the ssh command...')
                    instance1.sys("ssh -o StrictHostKeyChecking=no -i "
                                  + str(os.path.basename(instance1.keypath))
                                  + " root@" + str(instance2.private_ip_address)
                                  + " ' uname -a'", code=0)
                    self.debug('Ssh between instances passed')




    def test6_test_ssh_between_instances_in_diff_sec_groups_different_zone(self):
        '''
        Definition:
        This test attempts to set up security group rules between group1 and group2 to authorize group2 access
        from group1 across different zones.
        If no_cidr is True security groups will be setup using cidr notication ip/mask for each instance in
        group1, otherwise the entire source group 1 will authorized.
        the group will be
        Note: This test requires the CC have tunnelling enabled, or the CCs in each zone be on same
        layer 2 network segment.

        Test attempts to:
            -Authorize security groups for inter group private ip access.
            -Iterate through each zone and attempt to ssh from an instance in group1 to an instance in group2 over their
                private ips.
        '''
        zones = []
        if len(self.zones) < 2:
            raise SkipTestException('Skipping test5, only a single zone found or provided')
        self.status('Authorizing group2:' + str(self.group2.name) + ' for access from group1:' + str(self.group1.name))
        self.tester.authorize_group(self.group2, cidr_ip=None, port=None, src_security_group_name=self.group1.name)

        class TestZone():
            def __init__(self, zone):
                self.zone = zone
                self.test_instance_group1 = None
                self.test_instance_group2 = None

        for zone in self.zones:
            zones.append(TestZone(zone))


        self.debug('Grabbing  a single instance from each zone and from each test security group to use in this test...')
        for zone in zones:
            instance = None
            for instance in self.group1_instances:
                if instance.placement == zone.zone:
                    assert isinstance(instance, EuInstance)
                    zone.test_instance_group1 = instance
                    break
                instance = None
            if not zone.test_instance_group1:
                raise Exception('Could not find an instance in group1 for zone:' + str(zone.zone))
            instance = None
            for instance in self.group2_instances:
                if instance.placement == zone.zone:
                    assert isinstance(instance, EuInstance)
                    zone.test_instance_group2 = instance
                    break
            if not zone.test_instance_group2:
                raise Exception('Could not find instance in group2 for zone:' + str(zone.zone))
            instance = None

        self.status('Checking connectivity for instances in each zone, in separate but authorized security groups...')
        for zone in zones:
            instance1 = zone.test_instance_group1
            if not instance1:
                raise Exception('Missing instance in Security group: ' + str(self.group1.name) + ', Zone:' +
                                str(zone) + " = instance:" + str(instance1) )
            for zone2 in zones:
                if zone.zone != zone2.zone:
                    instance2 = zone2.test_instance_group2
                    if not instance2:
                        raise Exception('Missing instance in Security group: ' + str(self.group2.name) + ', Zone:' +
                                        str(zone2.zone) + " = instance:" + str(instance2) )
                    self.debug('Attempting to run ssh command "uname -a" between instances across zones and security groups:\n'
                               + str(instance1.id) + '/sec grps(' + str(instance1.security_groups)+") --> "
                               + str(instance2.id) + '/sec grps(' + str(instance2.security_groups)+")\n"
                               + "Current test run in zones: " + str(instance1.placement) + "-->" + str(instance2.placement),
                               linebyline=False )
                    self.debug('Check some debug information re this data connection in this security group first...')
                    self.tester.does_instance_sec_group_allow(instance=instance2,
                                                              src_addr=instance1.private_ip_address,
                                                              protocol='tcp',
                                                              port=22)
                    self.debug('Now Running the ssh command...')
                    instance1.sys("ssh -o StrictHostKeyChecking=no -i "
                                  + str(os.path.basename(instance1.keypath))
                                  + " root@" + str(instance2.private_ip_address)
                                  + " ' uname -a'", code=0)
                    self.debug('Ssh between instances passed')






if __name__ == "__main__":
    testcase = Net_Tests()

    ### Use the list of tests passed from config/command line to determine what subset of tests to run
    ### or use a predefined list

    if testcase.args.tests:
        list = testcase.args.tests
    else:
        list =['test1_create_instance_in_zones_for_security_group1',
               'test2_create_instance_in_zones_for_security_group2',
               'test3_test_ssh_between_instances_in_diff_sec_groups_same_zone',
               'test4_attempt_unauthorized_ssh_from_test_machine_to_group2',
               'test5_test_ssh_between_instances_in_same_sec_groups_different_zone',
               'test6_test_ssh_between_instances_in_diff_sec_groups_different_zone']
        ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )

    ### Run the EutesterUnitTest objects
    result = testcase.run_test_case_list(unit_list,eof=False,clean_on_exit=True)
    exit(result)






#!/usr/bin/env python
#
##########################
#                        #
#       Test Cases       #
#                        #
##########################
#
#        Attempts basic EBS related churn tests. Starting from basic to more involved churn tests as defined in testcase QA214:
#        Script repetitive runs stressing the following flows in concurrent use (Test script can be run concurrently via Jenkins, etc)
#        Each sub test can be called with --test testX or a list of tests, otherwise all tests will be run
#
#        test1) repeat: (create volume X -> delete volume X)
#        test2) create volume Y, then  repeat ( attach volume Y-> detach volume Y)
#        test3) repeat : ( create volume Z -> attach volume Z -> detach volume Z)
#        test4) repeat: (create volume V -> Create snapshot S from volume V, create volume SV from snapshot S, delete volume V, delete snapshot S, delete volume SV)
#        
#       
#        Cleanup:
#        --if 'fof' flag is not set, will remove all volumes, instance, and snapshots created during this test
#
#    @author: clarkmatthew



import unittest
from eutester.eutestcase import EutesterTestCase
from eutester.eutestcase import TestColor
from eucaops import Eucaops
import time



class Qa_214_volume_churn(EutesterTestCase):
    def __init__(self):
        #### Pre-conditions
        self.setuptestcase()
        self.setup_parser()
        self.parser.add_argument('--testcount', type=int, help='Number of times to repeat each test, default:5',default=5)
        self.parser.add_argument('--volcount', type=int, help='Number of volumes to create per test, default:5',default=5)
        self.parser.add_argument('--snapcount', type=int, help='Number of snapshots to create for snap related tests, default:2',default=2)
        self.parser.add_argument('--size', type=int, help='Size in GB for created volumes, default:1',default=1)
        self.parser.add_argument('--timepergig', type=int, help='Time allowed per gig size of volume during volume creation, default:120',default=120)
        self.parser.add_argument('--deletetimeout', type=int, help='Time allowed for volume to transition from deleting to deleted, default:120',default=120)
        self.get_args()
        # Setup basic eutester object
        self.tester = self.do_with_args(Eucaops)
        self.tester.debug = lambda msg: self.debug(msg, traceback=2, linebyline=False)
        self.reservation = None
        self.instance = None
        ### Add and authorize a group for the instance
        ### Generate a keypair for the instance
        #create some zone objects and append them to the zonelist
        self.zone = self.args.zone or 'PARTI00'
        self.groupname = 'jenkins'
        self.group = self.tester.add_group(self.groupname)
        self.tester.authorize_group(self.group)
        self.tester.authorize_group(self.group, protocol='icmp',port='-1')
        try:
            keys = self.tester.get_all_current_local_keys() 
            if keys != []:
                self.keypair = keys[0]
            else:
                self.keypair = keypair = self.tester.add_keypair('qa214volumechurn')
        except Exception, ke:
            raise Exception("Failed to find/create a keypair, error:" + str(ke))
       
        ### Get an image to work with
        self.image = self.tester.get_emi(root_device_type="instance-store")
        if not self.image:
            raise Exception('couldnt find image')
        self.clean_method = self.cleanup

    def cleanup(self, instances=True):
        '''
        if instances:
            try:
                if self.reservation:
                    self.tester.terminate_instances(self.reservation)
            except Exception, e:
                err = str(e)
        '''
        try:
            self.tester.cleanup_artifacts()
        except Exception, e:
            raise Exception('Cleanupfailed:'+str(e))
    
    def launch_test_instance(self):
        self.reservation = self.tester.run_instance(self.image, keypair=self.keypair, group=self.group,timeout=480)
        self.instance = self.reservation.instances[0]
        return self.instance
        
    def qa_214_test1(self, 
                     volcount=5,
                     testcount=20, 
                     size=1,  
                     timepergig=120, 
                     deletetimeout=120):
        """
        Definition: repeats: (create volume X -> delete volume X) 'count' number of times
        
        :param volcount: integer how many concurrent volumes to create/delete
        :param testcount: integer how many times to repeat this test
        :param size: integer size of volume(s) in GB
        :param timepergig: integer time allowed per GB in seconds during creation
        :param deletetimeout: integer timeout in seconds waiting for volume to transition to 'deleted' state
        """
        for x in xrange(0,testcount):
            self.status("\'qa_214_test1\' number:"+str(x)+"/"+str(testcount))
            self.volumes = self.tester.create_volumes(self.zone, size=size, count=volcount, timepergig=timepergig)
            self.tester.delete_volumes(self.volumes, poll_interval=5, timeout=deletetimeout)
            self.status('qa_214_test1: Completed:'+str(x)+'/'+str(testcount)+' tests',testcolor=TestColor.get_canned_color('whiteonblue'))
    
    
        
    def qa_214_test2(self, testcount=20, size=1,volcount=1):
        '''
        Definition: create volumes Y, then  repeat ( attach volumes Y-> detach volumes Y)
        
        :param testcount: integer times to run this test
        :param size: integer size of volume(s) to create in GB
        :param volcount: number of volumes to create for this test
        '''
        instance = self.instance or self.launch_test_instance()
        volumes = self.tester.create_volumes(self.zone, size=size, count=volcount)
        for x in xrange(0,testcount):
            self.status("\'qa_214_test2\' number:"+str(x)+"/"+str(testcount))
            for volume in volumes:
                instance.attach_volume(volume)
                time.sleep(1)
                instance.detach_euvolume(volume)
            self.status('qa_214_test2: Completed:'+str(x)+'/'+str(testcount)+' tests',testcolor=TestColor.get_canned_color('whiteonblue'))
                    
 
    def qa_214_test3(self, 
                     volcount=5,
                     testcount=5, 
                     size=1,  
                     timepergig=120, 
                     deletetimeout=120):
        """
        Definition:  repeat : ( create volume Z -> attach volume Z -> detach volume Z -> delete volume Z)
        
        :param volcount: integer how many concurrent volumes to create/delete
        :param testcount: integer how many times to repeat this test
        :param size: integer size of volume(s) in GB
        :param timepergig: integer time allowed per GB in seconds during creation
        :param deletetimeout: integer timeout in seconds waiting for volume to transition to 'deleted' state
        """
        instance = self.instance or self.launch_test_instance()
        for x in xrange(0,testcount):
            self.status("\'qa_214_test3\' number:"+str(x)+"/"+str(testcount))
            volumes = self.tester.create_volumes(self.zone, size=size, count=volcount, timepergig=timepergig)
            for volume in volumes:
                instance.attach_volume(volume)
                time.sleep(1)
                instance.detach_euvolume(volume)
            self.tester.delete_volumes(volumes, poll_interval=5, timeout=deletetimeout)
            self.status('qa_214_test3: Completed:'+str(x)+'/'+str(testcount)+' tests',testcolor=TestColor.get_canned_color('whiteonblue'))
            
    def qa_214_test4(self, 
                     volcount=5,
                     snapcount=2,
                     testcount=5, 
                     size=1,  
                     timepergig=120, 
                     deletetimeout=120):
        """
        Definition: repeat: (create volumes V -> Create snapshots S from volumes V,
                    create volume SV from snapshot S, delete volume V, 
                    delete snapshot S, delete volume SV)
        
        :param volcount: integer how many concurrent volumes to create/delete
        :param testcount: integer how many times to repeat this test
        :param size: integer size of volume(s) in GB
        :param timepergig: integer time allowed per GB in seconds during creation
        :param deletetimeout: integer timeout in seconds waiting for volume to transition to 'deleted' state
        """
        
        wait_on_progress = 15 * snapcount
        for x in xrange(0,testcount):
            volumes=[]
            snaps=[]
            self.status("\'qa_214_test4\' number:"+str(x)+"/"+str(testcount)+", volcount:"+str(volcount)+", snapcount:"+str(snapcount)+", size:"+str(size))
            self.status('Creating '+str(volcount)+' new volumes...')
            volumes = self.tester.create_volumes(self.zone, size=size, count=volcount, timepergig=timepergig)
            for vol in volumes:
                v_index = volumes.index(vol)+1
                self.status('Creating '+str(snapcount)+' snapshots from our new volume ('+str(v_index)+'/'+str(len(volumes))+'):'+str(vol.id) )
                snaps.extend(self.tester.create_snapshots(vol, count=snapcount, wait_on_progress=wait_on_progress, monitor_to_state=False))
            for snap in snaps:
                s_index = snaps.index(snap)+1
                self.status('Creating '+str(volcount)+' volumes from our new snapshot('+str(s_index)+'/'+str(len(snaps))+'):'+str(snap.id) )
                volumes.extend(self.tester.create_volumes(self.zone, count=volcount, snapshot=snap, timepergig=timepergig))
            self.status('Test#'+str(x)+': Main block of test complete, deleting '+str(len(volumes))+' volumes, and '+str(len(snaps))+' snapshots')
           
            self.tester.delete_volumes(volumes, poll_interval=5, timeout=deletetimeout)
            self.tester.delete_snapshot(snaps, basetimeout=deletetimeout)

            self.status('qa_214_test4: Completed:'+str(x)+'/'+str(testcount)+' tests',testcolor=TestColor.get_canned_color('whiteonblue'))
    

if __name__ == "__main__":
    testcase =Qa_214_volume_churn()
    
    ### Use the list of tests passed from config/command line to determine what subset of tests to run
    ### or use a predefined list
    list = testcase.args.tests or [ "qa_214_test1", "qa_214_test2", "qa_214_test3", "qa_214_test4"]

    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )

    ### Run the EutesterUnitTest objects
    result = testcase.run_test_case_list(unit_list,clean_on_exit=True)
    exit(result)



# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: vic.iglesias@eucalyptus.com

from eutester.eutestcase import EutesterTestCase
from multiprocessing import Process
from multiprocessing import Queue
import inspect
import uuid

class ProcessManager():
    def __init__(self):
        self.process_pool = {}
        self.queue_pool = {}

    def lookup_process(self, id):
        try:
            return self.process_pool[id]
        except KeyError,e:
            raise KeyError("Unable to find thread: " + str(id))

    def lookup_queue(self, id):
        try:
            return self.queue_pool[id]
        except KeyError,e:
            raise KeyError("Unable to find queue: " + str(id))

    def run_method_as_process(self, method, *args, **kwargs):
        methvars = EutesterTestCase.get_meth_arg_names(method)
        daemonize = False
        if 'daemonize' in kwargs:
            if 'daemonize' in methvars:
                daemonize = kwargs['daemonize']
            else:
                daemonize = kwargs.pop('daemonize')
        queue = Queue()
        id = uuid.uuid1().hex
        self.queue_pool[id] = queue
        process = Process(target=self.__run_method, args=(method, queue, self.__get_arguments(method,args,kwargs),))
        self.process_pool[id] = process
        process.daemon = daemonize
        process.start()
        return id

    def __run_method(self, method, queue, args):
        try:
            queue.put(method(**args))
        except Exception, e:
            queue.put(e)
            raise e

    def __get_arguments(self, func, args, kwargs):
        """
        Given a function and a set of arguments, return a dictionary of
        argument values that will be sent to the function.
        """
        arguments = {}
        spec = inspect.getargspec(func)
        if spec.defaults:
            arguments.update(zip(reversed(spec.args), reversed(spec.defaults)))
        if spec.keywords:
            arguments.update(spec.keywords)
        arguments.update(zip(spec.args, args))
        arguments.update(kwargs)
        return arguments

    def kill_process(self, id):
        thread = self.process_pool[id]
        assert isinstance(thread, Process)
        thread.terminate()
        self.remove_process(id)

    def remove_process(self, id):
        del self.process_pool[id]
        del self.queue_pool[id]

    def wait_for_process(self, id):
        process = self.lookup_process(id)
        process.join()
        return_value = self.queue_pool[id].get()
        self.remove_process(id)
        return return_value

    def get_all_results(self):
        result_list = []
        for process in self.process_pool:
                result_list.append(self.wait_for_process(process))
        return result_list




#!/usr/bin/python
# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: clarkmatthew

import eucaops
from eutester.eutestcase import EutesterTestCase
import time

class MyTestCase(EutesterTestCase):
    def __init__(self, config_file=None, password=None):
        self.setuptestcase()
        self.setup_parser()
        self.parser.add_argument("--timeout", default=600)
        self.get_args()


    def clean_method(self):
        self.debug('No clean_method defined for this test')
        pass

    def wait_for_services_operational(self, timeout=None):
        """
        Definition:
        Test attempts to query the state of a subset of core services. The test will continue to poll the system
        until it finds an ENABLED instance of each service. In the HA case it will wait for an ENABLED and DISABLED
        instance of each.
        """
        timeout= timeout or self.args.timeout
        last_err = ""
        elapsed = 0
        start = time.time()
        self.tester = None
        while (not self.tester and elapsed < timeout):
            elapsed = int(time.time() - start)
            self.status('Attempting to create tester object. Elapsed:' + str(elapsed))
            try:
                self.tester = eucaops.Eucaops(config_file=self.args.config_file, password=self.args.password)
            except Exception, e:
                tb = eucaops.Eucaops.get_traceback()
                last_err = str(tb) + "\n" + str(e)
                print 'Services not up because of: ' + last_err + '\n'
        if not self.tester:
            raise Exception(str(last_err) + 'Could not create tester object after elapsed:' + str(elapsed))
        timeout = timeout - elapsed
        self.status('starting wait for all services operational, timeout:' + str(timeout))
        self.tester.service_manager.wait_for_all_services_operational(timeout)
        self.status('All services are up')
        self.tester.service_manager.print_services_list()



if __name__ == "__main__":
    testcase = MyTestCase()
    ### Use the list of tests passed from config/command line to determine what subset of tests to run
    ### or use a predefined list  "VolumeTagging", "InstanceTagging", "SnapshotTagging", "ImageTagging"
    list = testcase.args.tests or ["wait_for_services_operational"]

    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )

    ### Run the EutesterUnitTest objects, dont worry about clean on exit until we need it for this method
    result = testcase.run_test_case_list(unit_list,clean_on_exit=False)
    exit(result)




# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: vic.iglesias@eucalyptus.com

import re
import time
from eutester import sshconnection
from eutester.sshconnection import SshCbReturn, CommandTimeoutException
import types
import stat
import eutester
import copy
import os
from eutester import machine
from xml.dom.minidom import parse, parseString
import dns.resolver


class log_marker:
    def __init__(self,
                 tester,
                 components,
                 start_marker=None,
                 log_files=None):
        """
        Manages unique markers placed in log/text files on a list of components.
        Intended to be used to indicate start and stop points of operations within the multiple sets of logs on multiple
        systems. These log segments can then collected, analyzed, etc. at a later point in time.

        :param tester: eutester object
        :param components: list of component objs to operate on
        :param start_marker: unique string, this will be auto-generated if not provided.
        :param log_files: list of files (using full path to files to mark/operate upon on remote components

        """
        self.tester = tester
        self.components = components
        self.start_marker = start_marker or "eutester_marker_start:" + \
                                         str(time.time()) + str(self.tester.id_generator(size=10))
        self.log_files = log_files
        self.start_time = time.time()
        self.end_marker = None
        self.end_time = None
        self.elapsed = None


    def add_start_marker(self, components=None, log_files=None, marker=None):
        """
        Attempts to add a unique 'marker' in the 'log_files' provided in all the 'components' provided.

        :param components:
        :param log_files:
        :param marker:
        """
        marker = marker or self.start_marker
        log_files = log_files or self.log_files
        components = components or self.components

        for component in components:
            if component.machine:
                log_files = log_files or component.log_files

        return marker



class Eunode:
    def __init__(self,
                 tester,
                 hostname,
                 partition,
                 name=None,
                 instance_ids=None,
                 instances=None,
                 state = None,
                 machine = None,
                 debugmethod = None,
                 ):
        """
        init object containing node related info and methods

        :param hostname: mandatory - string - hostname of the node (ie: 192.168.1.1)
        :param partition: mandatory - string - partition name of the node (ie PARTI00)
        :param name:  optional string - optional name for this eunode. Defaults to hostname
        :param instances: - optional -list of instance strings reported on this node
        :param machine: optional eutester machine type object
        :param tester: - eutester obj
        """
        type = EuserviceManager.node_type_string
        self.hostname = hostname
        self.partition = partition
        self.part_name = partition.name
        self.name = name or self.hostname
        self.instance_ids = instance_ids or []
        self.state = state
        self.tester = tester
        self.machine = machine
        self.service_state = None
        self.debugmethod = debugmethod or self.tester.debug

        if not machine:
            try:
                self.machine = self.tester.get_machine_by_ip(hostname)
                self.get_service_state()
            except Exception, e:
                self.debug("Failed to get machine for this node:" + str(hostname) + ", err:" + str(e))
        #if self.machine:
            #self.hypervisor =


    def debug(self, msg):
        """
        Simple method to print debug messsage 'msg'
        :param msg: message to be printed
        """
        if self.debugmethod:
            self.debugmethod(msg)
        else:
            print(msg)


    def sys(self, cmd, code=None):
        """
        Command to be executed via ssh on remote eunode machine
        :param cmd: string - command to be executed
        :param code: int - optional exit code used to determine pass fail of remote command.
        :return: list of lines from remote cmd's output
        """
        return self.machine.sys(cmd,code=code)

    def stop(self):
        self.sys(self.tester.eucapath + "/etc/init.d/eucalyptus-nc stop", code=0)

    def start(self):
        self.sys(self.tester.eucapath + "/etc/init.d/eucalyptus-nc start", code=0)

    def get_hypervisor_from_euca_conf(self):
        """
        Attempts to find HYPERVISOR value in <eucalytpus home>/etc/eucalyptus.conf

        :return: string representing hypervisor type if found
        """
        hypervisor =  None
        out = self.sys("cat /etc/eucalyptus/eucalyptus.conf | grep '^HYPER'")
        if out and re.search('^HYPERVISOR=',out[0]):
            hypervisor = out[0].split('=')[1].strip().strip('"')
        return hypervisor


    def get_service_state(self):
        service_state = None
        if self.machine:
            try:
                if self.machine.distro.name is not "vmware":
                    self.sys("service eucalyptus-nc status", code=0)
                    service_state = 'running'
                else:
                    service_state = 'running'
            except sshconnection.CommandExitCodeException:
                service_state = 'not_running'
            except Exception, e:
                self.debug('Could not get service state from node:' + str(self.hostname) + ", err:"+str(e))
        else:
            print "No machine object for this eunode:" + str(self.hostname)
        self.service_state = service_state
        return service_state


    def get_virsh_list(self):
        """
        Return a dict of virsh list domains.
        dict should have dict['id'], dict['name'], dict['state']

        """
        instance_list = []
        if self.machine:
            keys = []
            output = self.machine.sys('virsh list', code=0)
            if len(output) > 1:
                keys = str(output[0]).strip().lower().split()
                for line in output[2:]:
                    line = line.strip()
                    if line == "":
                        continue
                    domain_line = line.split()
                    instance_list.append({keys[0]:domain_line[0], keys[1]:domain_line[1], keys[2]:domain_line[2]})
        return instance_list

    def tail_instance_console(self,
                              instance,
                              max_lines=None,
                              timeout=30,
                              idle_timeout=30,
                              print_method=None):
        '''


        '''
        if timeout < idle_timeout:
            idle_timeout = timeout
        if not isinstance(instance,types.StringTypes):
            instance = instance.id
        console_path = self.get_instance_console_path(instance)
        start_time = time.time()
        lines_read = 0
        print_method = print_method or self.debug
        prefix = str(instance) + " Console Output:"
        try:
            self.machine.cmd('tail -F ' + str(console_path),
                             verbose=False,
                             cb=self.remote_tail_monitor_cb,
                             cbargs=[instance,
                                     max_lines,
                                     lines_read,
                                     start_time,
                                     timeout,
                                     print_method,
                                     prefix,
                                     idle_timeout],
                             timeout=idle_timeout)
        except CommandTimeoutException, cte:
            self.debug('Idle timeout fired while tailing console: ' + str(cte))


    def remote_tail_monitor_cb(self,
                               buf,
                               instance_id,
                               max_lines,
                               lines_read,
                               start_time,
                               timeout,
                               print_method,
                               prefix,
                               idle_timeout):
        ret = SshCbReturn(stop=False, settimer=idle_timeout)
        return_buf = ""
        now = time.time()
        if (timeout and (now - start_time) >= timeout) or (max_lines and lines_read >= max_lines):
            ret.statuscode = 0
            ret.stop = True
        try:
            for line in str(buf).splitlines():
                lines_read += 1
                print_method(str(prefix) + str(line))
        except Exception, e:
            return_buf = "Error in remote_tail_monitor:" + str(e)
            ret.statuscode = 69
            ret.stop = True
        finally:
            ret.buf = return_buf
            ret.nextargs = [instance_id, max_lines, lines_read, start_time,timeout]
            return ret


    def get_instance_multipath_dev_info_for_instance_ebs_volume(self, instance, volume):
        if not isinstance(instance,types.StringTypes):
            instance = instance.id
        if isinstance(volume,types.StringTypes):
            volume = self.tester.get_volume(volume_id=volume)
        if volume.attach_data and volume.attach_data.instance_id == instance:
            dev = volume.attach_data.device
        else:
            raise Exception(str(volume.id) + 'Vol not attached to instance: ' + str(instance))
        return self.get_instance_multipath_dev_info_for_instance_block_dev(instance, dev)


    def get_instance_multipath_dev_info_for_instance_block_dev(self, instance, ebs_block_dev, verbose=False):
        if not isinstance(instance,types.StringTypes):
            instance = instance.id
        mpath_dev = self.get_instance_multipath_dev_for_instance_block_dev(instance, ebs_block_dev)
        mpath_dev_info = self.machine.sys('multipath -ll ' + str(mpath_dev) + " | sed 's/[[:cntrl:]]//g' ",
                                          verbose=verbose, code=0)
        return mpath_dev_info

    def get_instance_multipath_dev_for_instance_ebs_volume(self, instance, volume):
        if not isinstance(instance,types.StringTypes):
            instance = instance.id
        if isinstance(volume,types.StringTypes):
            volume = self.tester.get_volume(volume_id=volume)

    def get_instance_multipath_dev_for_instance_block_dev(self, instance, ebs_block_dev, verbose=False):
        mpath_dev = None
        ebs_block_dev = os.path.basename(ebs_block_dev)
        if not isinstance(instance,types.StringTypes):
            instance = instance.id
        dm_dev = self.get_instance_block_disk_dev_on_node(instance, ebs_block_dev)
        sym_links = self.machine.sys('udevadm info --name ' + str(dm_dev) + ' --query symlink',
                                     verbose=verbose, code=0)[0]
        for path in str(sym_links).split():
            if str(path).startswith('mapper/'):
                mpath_dev = path.split('/')[1]
                break
        return mpath_dev


    def get_instance_block_disk_dev_on_node(self, instance, block_dev):
        block_dev = os.path.basename(block_dev)
        if not isinstance(instance,types.StringTypes):
            instance = instance.id
        paths = self.get_instance_block_disk_source_paths(instance)
        sym_link  = paths[block_dev]
        real_dev = self.machine.sys('readlink -e ' + sym_link, verbose=False, code=0)[0]
        fs_stat = self.machine.get_file_stat(real_dev)
        if stat.S_ISBLK(fs_stat.st_mode):
            return real_dev
        else:
            raise(str(instance) + ", dev:" + str(block_dev) + ',Error, device on node is not block type :' + str(real_dev))

    def get_instance_block_disk_source_paths(self, instance, target_dev=None):
        '''
        Returns dict mapping target dev to source path dev/file on NC
        Example return dict: {'vdb':'/NodeDiskPath/dev/sde'}
        '''
        ret_dict = {}
        if target_dev:
            target_dev = os.path.basename(target_dev)
        if not isinstance(instance,types.StringTypes):
            instance = instance.id
        disk_doms = self.get_instance_block_disk_xml_dom_list(instance_id=instance)
        for disk in disk_doms:
            source_dev = disk.getElementsByTagName('source')[0].attributes.get('dev').nodeValue
            target_bus = disk.getElementsByTagName('target')[0].attributes.get('dev').nodeValue
            if not target_dev or target_dev == target_bus:
                ret_dict[target_bus] = str(source_dev)
        return ret_dict

    def get_instance_console_path(self, instance_id):
        if not isinstance(instance_id,types.StringTypes):
            instance = instance_id.id
        dev_dom = self.get_instance_device_xml_dom(instance_id=instance_id)
        console_dom = dev_dom.getElementsByTagName('console')[0]
        return console_dom.getElementsByTagName('source')[0].attributes.get('path').nodeValue


    def get_instance_device_xml_dom(self, instance_id):
        if not isinstance(instance_id,types.StringTypes):
            instance = instance_id.id
        dom = self.get_instance_xml_dom(instance_id)
        return dom.getElementsByTagName('devices')[0]

    def get_instance_block_disk_xml_dom_list(self, instance_id):
        if not isinstance(instance_id,types.StringTypes):
            instance = instance_id.id
        dev_dom = self.get_instance_xml_dom(instance_id)
        return dev_dom.getElementsByTagName('disk')

    def get_instance_xml_dom(self, instance_id):
        if not isinstance(instance_id,types.StringTypes):
            instance = instance_id.id
        output = self.get_instance_xml_text(instance_id)
        dom_xml = parseString(output)
        return dom_xml.getElementsByTagName('domain')[0]

    def get_instance_xml_text(self, instance_id):
        if not isinstance(instance_id,types.StringTypes):
            instance = instance_id.id
        return self.machine.sys('virsh dumpxml ' + str(instance_id),listformat=False, verbose=False, code=0)


    #def get_iscsi_connections(self,):
    #def get_exported_volumes(self,)
    #def get_all_instances_on_node(self, instance_id=None, state=None)



class Euservice(object):

    activeactive_service_types = ["objectstorage"]
    
    def __init__(self, service_string, tester = None):
        values = service_string.split()
        self.type = values[1]
        self.partition = values[2]
        self.name = values[3]
        self.state = values[4]
        self.uri = values[6]
        self.fullname = values[7]
        self.hostname = self.uri.split(":")[1].split("/")[2]
        self.running = True
        self.tester = tester
        self.machine = None
        if self.tester:
            self.machine = tester.get_machine_by_ip(self.hostname)
        
    def isEnabled(self):
        if re.search('ENABLED',self.state) :
            return True
        else:
            return False
        
    def isDisabled(self):
        if not re.search('ENABLED', self.state):
            return True
        else:
            return False
    
    def isActiveActive(self):
        return self.type in Euservice.activeactive_service_types
        
    def disable(self):
        self.tester.service_manager.disable(self)
        
    def enable(self):
        self.tester.service_manager.enable(self)
    
    def stop(self):
        self.tester.service_manager.stop(self)
    
    def start(self):
        self.tester.service_manager.start(self)

    def get_service_string(self):
        if self.type == 'cluster':
            return 'eucalyptus-cc'
        else:
            return 'eucalyptus-cloud'

    def print_self(self, header=True, footer=True, printmethod=None):
        part_len = 16 #self.partition
        type_len  = 16 #self.type
        hostname_len = 24 #self.hostname
        state_len = 10 #self.state
        uri_len = 36 #self.uri
        buf = "\n"
        line = "------------------------------------------------------------------------------------------------------------------------------------------"
        if header:
            buf += str(line+"\n")
            buf += str('PARTITION').center(part_len)+'|'+str('TYPE').center(type_len)+'|'+str('HOSTNAME').center(hostname_len)+'|'+str('STATE').center(state_len)+'|'+str('URI').center(uri_len)+'\n'
            buf += str(line+"\n")
        buf += str(self.partition).center(part_len)+'|'+str(self.type).center(type_len)+'|'+str(self.hostname).center(hostname_len)+'|'+str(self.state).center(state_len)+'|'+str(self.uri).center(uri_len)
        if footer:
            buf += str("\n"+ line)
        if printmethod:
            printmethod(buf)
        return buf

    @staticmethod
    def create_service(service_string, tester=None):
        if service_string.split()[1] == 'dns':
            return DnsService(service_string)
        else:
            return Euservice(service_string, tester)



class DnsService(Euservice):
    def __init__(self, service_string):
        super(DnsService, self).__init__(service_string)
        self.resolver = dns.resolver.Resolver(configure=False)
        self.resolver.nameservers = [self.hostname]

    def resolve(self, name, timeout=360, poll_count=20):
        """Resolve hostnames against the Eucalyptus DNS service"""
        poll_sleep = timeout/poll_count
        for _ in range(poll_count):
            try:
                print("DNSQUERY: Resolving `{0}' against nameserver(s) {1}".format(name, self.resolver.nameservers))
                ans = self.resolver.query(name)
                return str(ans[0])
            except dns.resolver.NXDOMAIN:
                raise Exception("Unable to resolve hostname `{0}'".format(name))
            except dns.resolver.NoNameservers:
                # Note that this usually means our DNS server returned a malformed message
                pass
            finally:
                time.sleep(poll_sleep)
        raise Exception("Unable to resolve hostname `{0}'".format(name))


        
class Partition:

    
    def __init__(self, name, service_manager ):
        self.name = name
        self.service_manager = service_manager
        name = ""
        self.ccs = []
        self.scs = []
        self.vbs = []
        self.ncs = []
        self.volumes = []
        self.instances = []
        
    def get_enabled(self, list):
        self.service_manager.update()
        for service in list:
            if service.isEnabled():
                return service
        raise Exception("Unable to find an enabled service in: " + str(list))
    
    def get_disabled(self, list):
        self.service_manager.update()
        for service in list:
            if not service.isEnabled():
                return service
        raise Exception("Unable to find an disabled service in: " + str(list))
    
    def get_enabled_cc(self):
        return self.get_enabled(self.ccs)
    
    def get_enabled_sc(self):
        return self.get_enabled(self.scs)
    
    def get_enabled_vb(self):
        return self.get_enabled(self.vbs)
    
    def get_disabled_cc(self):
        return self.get_disabled(self.ccs)
    
    def get_disabled_sc(self):
        return self.get_disabled(self.scs)
    
    def get_disabled_vb(self):
        return self.get_disabled(self.vbs)
 
class EuserviceManager(object):
    cluster_type_string = "cluster"
    walrus_type_string = 'walrus'
    osg_type_string = 'osg'
    storage_type_string = 'storage'
    clc_type_string = 'eucalyptus'
    node_type_string = 'node'
    osg_type_string='objectstorage'

        
    def __init__(self, tester ):
        '''
        SERVICE    storage            PARTI00            SC_61              ENABLED       16      http://192.168.51.32:8773/services/Storage    arn:euca:eucalyptus:PARTI00:storage:SC_61/
        update this service based up on the information parsed out of the "describestring"
        '''
        ### Make sure i have the right connection to make first contact with euca-describe-services
        self.walruses= []
        self.osgs = []
        self.clcs = []
        self.arbitrators = []
        self.partitions = {}
        self.internal_components = []
        self.dns = None
        self.all_services = []
        self.node_list = []
        self.tester = tester
        self.debug = tester.debug
        self.eucaprefix = ". " + self.tester.credpath + "/eucarc && " + self.tester.eucapath
        if self.tester.clc is None:
            raise AttributeError("Tester object does not have CLC machine to use for SSH")
        self.last_updated = None
        self.update()

    @eutester.Eutester.printinfo
    def get(self, type=None, partition=None, attempt_both=True, poll_interval=15, allow_clc_start_time=300):
        """
        Method attempts to 'get' euservices by parsing euca-describe-services on the CLC(s). The method
        will do some basic service state checks as well as wait for a reasonable amount
        of time 'allow_clc_start_time' in seconds to allow the eucalyptus service(s) to initialize/sync and be ready
        to service requests.

        :param type: service type string to filter returned services list by
        :param partition: partition, aka zone, aka cluster to filter by.
        :param attempt_both: When set, query the alternate CLC if the first errors
        :param allow_clc_start_time: In the case both CLCs are not responding, this is used as the timeout. This time
                                     represents the amount of time allowed from the time the clc process(s) were started.
                                     If a valid response is not detected, this method will error out.
        :return: list of euservices
        """

        good_clc_hosts = []
        describe_services = []
        services = []
        clc_hostnames = ""
        err_msg = ""
        dbg_msg = ""

        if type is not None:
            type = " -T " + str(type) 
        else:
            type = ""
        #### This is a hack around the fact that the -P filter is not working need to fix this once that functionality is fixed
        if partition is None:
            partition = ""
        #Check each CLC to make sure the eucalyptus-cloud service is running on that machine
        if attempt_both:
            clc_machines =self.tester.get_component_machines("clc")
        else:
            clc_machines =[self.tester.clc]

        for clc_machine in clc_machines:
            clc_hostnames += clc_machine.hostname + ","
            if clc_machine.get_eucalyptus_cloud_is_running_status():
                #Add to front of check list if this is the current tester clc, otherwise append to end.
                if clc_machine == self.tester.clc:
                    good_clc_hosts.insert(0,clc_machine)
                else:
                    good_clc_hosts.append(clc_machine)
            else:
                self.debug('Eucalyptus cloud was not found running on CLC:'+str(clc_machine.hostname))

        for clc_host in good_clc_hosts:
            dbg_msg += clc_host.hostname + ", "
        self.debug("Checking the following CLCs for services/status: " + str(dbg_msg) + "...")
        while good_clc_hosts and not describe_services:
            clc_process_uptimes = []
            process_uptime = None
            for clc in good_clc_hosts:
                try:
                    #Save proess uptime for potential debug if request fails.
                    process_uptime = self.tester.clc.get_eucalyptus_cloud_process_uptime()
                    #Store all CLC's process uptimes in list, compare for youngest later...
                    clc_process_uptimes.append(process_uptime)
                    out = clc.sys(self.eucaprefix + "/usr/sbin/euca-describe-services " + str(type), code=0,timeout=15)
                    for line in out:
                        if re.search("SERVICE.+"+str(partition), line):
                            describe_services.append(line)
                    if not describe_services:
                        raise IndexError("Did not receive proper response from describe services when looking for " + str(type))
                    else:
                        #if the check was successful, we may need to swap the tester's primary clc
                        if clc != self.tester.clc:
                            self.tester.swap_clc()
                        break
                except Exception, e:
                    self.debug("Did not get a valid response from clc:" + str(clc.hostname) +", err:" +str(e))
                    err_msg += str(e) + "\n"
                    # Check to make sure the CLC process is evening running on this machine, and if the youngest CLC process
                    # has been up for a reasonable amount of time to sync and/or service requests.
                    is_running = self.tester.clc.get_eucalyptus_cloud_is_running_status()
                    if not is_running:
                        good_clc_hosts.remove(self.tester.clc)
                        err_msg += "Error in service request on CLC:" + str(self.tester.clc.hostname) \
                                   + ". PID uptime:" + str(process_uptime) + "/" + str(allow_clc_start_time) \
                                   + ", service_running:"+ str(is_running) + "\n"
            #If we've checked both CLCs and still don't have a valid response to parse, 'and' the youngest CLC
            #process uptime has exceeed 'allow_clc_start_time' then raise error.
            if not good_clc_hosts or \
                    not clc_process_uptimes or \
                    (not describe_services and min(clc_process_uptimes) > allow_clc_start_time):
                raise Exception("Could not get services from " + str(clc_hostnames)
                                + ", after clc process uptime of at least "
                                + str(allow_clc_start_time) + "\nErrors:"+str(err_msg))
            if not describe_services:
                time.sleep(poll_interval)
        #Create euservice objects from command output and return list of euservices.
        for service_line in describe_services:
            services.append(Euservice.create_service(service_line, self.tester))
        return services

    def print_services_list(self, services=None):
        services = services or self.all_services
        services_list = copy.copy(services)
        service1 = services_list.pop(0)
        buf = service1.print_self()
        for service in services_list:
            buf += service.print_self(header=False,)
        self.debug(buf)



    def reset(self):
        self.walruses= []
        self.osgs = []
        self.clcs = []
        self.arbitrators = []
        self.internal_components = []
        for k, v in self.partitions.iteritems():
            self.partitions[k].ccs = []
            self.partitions[k].scs = []
            self.partitions[k].vbs = []


    def compare_versions(self, version1, version2):
        '''
        :param version1:
        :param version2:
        :returns: 1 - if version1 is newer than version2
        :returns: 0 - if versions are equal
        :returns: -1 - if version1 is older than version2
        '''
        ver1 = str(version1).split('.')
        ver2 = str(version2).split('.')
        if not ver1 or not ver2:
            raise Exception('Failed to parse versions from strings:' + str(version1) + ", " + str(version2))
        while ver1:
            if not len(ver2):
                return 1
            sub_ver1 = int(ver1.pop(0))
            sub_ver2 = int(ver2.pop(0))
            if sub_ver1 > sub_ver2:
                return 1
            if sub_ver1 < sub_ver2:
                return -1
        #version 1 has no additional sub release ids, and until this point ver1 == ver2...
        while ver2:
            if int(ver2.pop(0)) != 0:
                #ver2 has a none '0' sub release id so it is > than ver1
                return -1
        #versions are equal
        return 0


    def populate_nodes(self, enabled_clc=None):
        """
        Sort output of 'list nodes cmd' on clc, create/update eunode objects.
        Returned list is used to update:'service_manager.node_list'

        :param enabled_clc: To avoid an update() or update() loop the current enabled clc can be provided. This can
                            also be used to test nc lookup on disabled CLC by providing this component obj instead.
        :return: list of eunode objects
        """
        return_list = []
        #to avoid update() loop allow enabled_clc to be provided as arg
        clc = enabled_clc or self.get_enabled_clc()
        clc_version = clc.machine.get_eucalyptus_version()
        if self.compare_versions(clc_version,'3.3') >= 0:
            try:
                return self.populate_nodes_3_3(enabled_clc)
            except:
                return self.populate_nodes_pre_3_3(enabled_clc)
        else:
            return self.populate_nodes_pre_3_3(enabled_clc)


    def populate_nodes_3_3(self, enabled_clc=None):
        """
        Sort output of 'list nodes cmd' on clc, create/update eunode objects.
        Returned list is used to update:'service_manager.node_list'

        :param enabled_clc: To avoid an update() or update() loop the current enabled clc can be provided. This can
                            also be used to test nc lookup on disabled CLC by providing this component obj instead.
        :return: list of eunode objects

        version >= 3.3.0 output (note state and instances on lines to follow node(s)
        [type]  [partition]     [node hostname] [state]
        NODE    PARTI00         192.168.51.15   ENABLED
        NODE    PARTI00         192.168.51.13   ENABLED
        [type]           [instances per line]
        INSTANCE        i-A1BE4281
        """
        type_loc = 0
        partition_loc = 1
        hostname_loc = 2
        state_loc = 3
        instance_id_loc = 1
        return_list = []
        instance_list = []
        last_node = None
        #to avoid update() loop allow enabled_clc to be provided as arg
        clc = enabled_clc or self.get_enabled_clc()
        nodes_strings = clc.machine.sys(self.eucaprefix + \
                                        "/usr/sbin/euca_conf --list-nodes 2>1 | grep 'NODE\|INSTANCE'")
        for node_string in nodes_strings:
            #handle/skip any blank lines first...
            node_string = node_string.strip()
            if not node_string:
                continue
            partition = None
            #sort out the node string...
            split_string = node_string.split()
            if split_string[type_loc] == 'INSTANCE':
                node.instance_ids.append(split_string[instance_id_loc])
            elif(split_string[type_loc] == 'NODE'):
                hostname = split_string[hostname_loc]
                partition_name = split_string[partition_loc]
                state = split_string[state_loc]
                # grab the list of instances if any found in the string
                #Try to match the part_name to the partition name it resides in
                for part in self.get_all_partitions():
                    if part.name == partition_name:
                            partition = part
                            break
                if not partition:
                    raise Exception('populate_nodes: Node:' + str(hostname) + ' Failed to find partition for name: '
                                    + str(partition_name))
                node = Eunode(self.tester,
                              hostname,
                              partition,
                              state = state)
                return_list.append(node)
                if node in part.ncs:
                    part.ncs[part.ncs.index(node)]=node
                else:
                    part.ncs.append(node)
        self.node_list = return_list
        return return_list


    def populate_nodes_pre_3_3(self,enabled_clc=None):
        """
        Sort output of 'list nodes cmd' on clc, create/update eunode objects.
        Returned list is used to update:'service_manager.node_list'

        :param enabled_clc: To avoid an update() or update() loop the current enabled clc can be provided. This can
                            also be used to test nc lookup on disabled CLC by providing this component obj instead.
        :return: list of eunode objects

        output for <= 3.2.2
        [type]  [node hostname] [cc_name] [instances....]
        NODE	192.168.51.72	CC_71	i-9A293E9B
        """
        type_loc = 0
        hostname_loc = 1
        cc_name_loc = 2
        instances_loc = 3
        return_list = []
        #to avoid update() loop allow enabled_clc to be provided as arg
        clc = enabled_clc or self.get_enabled_clc()
        nodes_strings = clc.machine.sys(self.eucaprefix + \
                                        "/usr/sbin/euca_conf --list-nodes 2>/dev/null | grep '^NODE'")

        for node_string in nodes_strings:
            #handle/skip any blank lines first...
            self.debug('Handling Node string:' + str(node_string))
            node_string = node_string.strip()
            if not node_string:
                continue
            instance_list = []
            partition = None
            #sort out the node string...
            split_string = node_string.split()
            cc_name = split_string[cc_name_loc]
            hostname = split_string[hostname_loc]
            # grab the list of instances if any found in the string
            if len(split_string) > instances_loc:
                instance_list = split_string[instances_loc:]
                #Try to match the part_name to the partition name it resides in
            for part in self.get_all_partitions():
                for cc in part.ccs:
                    if cc.name == cc_name:
                        partition = part
                        break
            if not partition:
                raise Exception('populate_nodes: Node:' + str(hostname) + ' Failed to find partition for component: '
                                + str(cc_name))
            node = Eunode(self.tester,
                          hostname,
                          partition,
                          instance_ids = instance_list,
                          state = 'ENABLED')
            return_list.append(node)
            if node in part.ncs:
                part.ncs[part.ncs.index(node)]=node
            else:
                part.ncs.append(node)
        self.node_list = return_list
        return return_list


    def update_node_list(self, enabled_clc=None):
        self.populate_nodes(enabled_clc=enabled_clc)

    def update_service_list(self):
        return self.get()

    @eutester.Eutester.printinfo
    def get_all_services_by_filter(self,
                                   type=None,
                                   partition=None,
                                   state=None,
                                   name=None,
                                   hostname=None,
                                   running=None,
                                   use_cached_list=True):
        """
        Returns a list of services of service that match the provided filter criteria.

        :param type: type of service to look for (cluster, eucalyptus, storage, walrus)
        :param partition: partition to filter returned service list with
        :param state: state to filter returned service list with
        :param name: name to filter returned service list with
        :param hostname: hostname/ip to filter returned service list with
        :param running: running boolean to filter returned service list with
        :param use_cached_list: use current list and state of self.all_services, else get_all_services()
        :return: list of matching cc euservice objects

        Examples:
                - Get all the services running on a specific machine "XYZ":
                    get_all_services_by_filter(hostname="XYZ")
                - Get all services in ENABLED state:
                    get_all_services_by_filter(state="ENABLED")
                - Get all Storage controllers in zone/partition "MYZONE":
                    get_all_services_by_filter(type=self.storage_type_string, partition="MYZONE")
        """
        euservice_list = []
        if use_cached_list:
            services = self.all_services or self.get_all_services()
        else:
            services = self.get_all_services()
        for service in services:
                if type and service.type != type:
                    continue
                if partition and service.partition != partition:
                    continue
                if state and service.state != state:
                    continue
                if name and service.name != name:
                    continue
                if hostname and service.hostname != hostname:
                    continue
                if running is not None and service.running != running:
                    continue
                euservice_list.append(service)
        return euservice_list


    def get_all_cluster_controllers(self,
                                    partition=None,
                                    state=None,
                                    name=None,
                                    hostname=None,
                                    running=None,
                                    use_cached_list=True):
        """
        Returns a list of services of cluster services that match the provided filter criteria.

        :param partition: partition to filter returned service list with
        :param state: state to filter returned service list with
        :param name: name to filter returned service list with
        :param hostname: hostname/ip to filter returned service list with
        :param running: running boolean to filter returned service list with
        :param service_list: list of euservices to filter from
        :param use_cached_list: use current list and state of self.all_services, else get_all_services()
        :return: list of matching cc euservice objects
        """
        return self.get_all_services_by_filter(type=self.cluster_type_string,
                                                 partition=partition,
                                                 state=state,
                                                 name=name,
                                                 hostname=hostname,
                                                 running=running,
                                                 use_cached_list=use_cached_list)

    def get_all_storage_controllers(self,
                                    partition=None,
                                    state=None,
                                    name=None,
                                    hostname=None,
                                    running=None,
                                    use_cached_list=True):
        """
        Returns a list of services of storage controller services that match the provided filter criteria.

        :param partition: partition to fileter returned service list with
        :param state: state to filter returned service list with
        :param name: name to filter returned service list with
        :param hostname: hostname/ip to filter returned service list with
        :param running: running boolean to filter returned service list with
        :param service_list: list of euservices to filter from
        :param use_cached_list: use current list and state of self.all_services, else get_all_services()
        :return: list of matching cc euservice objects
        """
        return self.get_all_services_by_filter(type=self.storage_type_string,
                                             partition=partition,
                                             state=state,
                                             name=name,
                                             hostname=hostname,
                                             running=running,
                                             use_cached_list=use_cached_list)

    def get_all_walrus(self,
                       partition=None,
                       state=None,
                       name=None,
                       hostname=None,
                       running=None,
                       use_cached_list=True):
        """
        Returns a list of services of walrus services that match the provided filter criteria.

        :param partition: partition to fileter returned service list with
        :param state: state to filter returned service list with
        :param name: name to filter returned service list with
        :param hostname: hostname/ip to filter returned service list with
        :param running: running boolean to filter returned service list with
        :param service_list: list of euservices to filter from
        :param use_cached_list: use current list and state of self.all_services, else get_all_services()
        :return: list of matching cc euservice objects
        """
        return self.get_all_services_by_filter(type=self.walrus_type_string,
                                             partition=partition,
                                             state=state,
                                             name=name,
                                             hostname=hostname,
                                             running=running,
                                             use_cached_list=use_cached_list)

    def get_all_osgs(self, partition=None,
                                  state=None,
                                  name=None,
                                  hostname=None,
                                  running=None,
                                  use_cached_list=True):
        """
        Returns a list of Object Storage Gateways that match the provided filter criteria.

        :param partition: partition to filter returned service list with
        :param state: state to filter returned service list with
        :param name: name to filter returned service list with
        :param hostname: hostname/ip to filter returned service list with
        :param running: running boolean to filter returned service list with
        :param service_list: list of euservices to filter from
        :param use_cached_list: use current list and state of self.all_services, else get_all_services()
        :return: list of matching cc euservice objects
        """
        return self.get_all_services_by_filter(type=self.osg_type_string,
                                             partition=partition,
                                             state=state,
                                             name=name,
                                             hostname=hostname,
                                             running=running,
                                             use_cached_list=use_cached_list)

    def get_all_cloud_controllers(self,
                                 partition=None,
                                 state=None,
                                 name=None,
                                 hostname=None,
                                 running=None,
                                 use_cached_list=True):
        """
        Returns a list of services of CLC services that match the provided filter criteria.

        :param partition: partition to fileter returned service list with
        :param state: state to filter returned service list with
        :param name: name to filter returned service list with
        :param hostname: hostname/ip to filter returned service list with
        :param running: running boolean to filter returned service list with
        :param service_list: list of euservices to filter from
        :param use_cached_list: use current list and state of self.all_services, else get_all_services()
        :return: list of matching cc euservice objects
        """
        return self.get_all_services_by_filter(type=self.clc_type_string,
                                             partition=partition,
                                             state=state,
                                             name=name,
                                             hostname=hostname,
                                             running=running,
                                             use_cached_list=use_cached_list)

    def get_all_node_controllers(self,
                                 hostname=None,
                                 partition=None,
                                 part_name=None,
                                 instance_id=None,
                                 has_instances=None,
                                 service_state=None,
                                 use_cached_list=True):
        """
        Returns a list of services of node controllers that match the provided filter criteria.

        :param part_name: name of partition to filter list with
        :param instance_id: filter list for specific instance id
        :param has_instances: filter list by nodes which have instance ids associated with them
        :param partition: partition obj to filter returned service list with
        :param name: name to filter returned service list with
        :param hostname: hostname/ip to filter returned service list with
        :param running: running boolean to filter returned service list with
        :param use_cached_list: use current list and state of self.all_services, else get_all_services()
        :return: list of matching cc euservice objects
        """
        return_list = []
        if use_cached_list:
            nodes = self.node_list or self.populate_nodes()
        else:
            nodes = self.populate_nodes()
        for node in nodes:
            if partition and node.partition != partition:
                continue
            if part_name and node.part_name != part_name:
                continue
            if hostname and node.hostname != hostname:
                continue
            if instance_id and not instance_id in node.instance_ids:
                continue
            if service_state and service_state != node.service_state:
                continue
            if has_instances and not node.instance_ids:
                continue
            return_list.append(node)
        return return_list


    def get_all_partitions(self):
        return_list = []
        for key in self.partitions:
            part = self.partitions[key]
            return_list.append(part)
        return return_list

    def get_all_services(self):
        all_services = []
        self.update()
        if len(self.clcs) > 0:
            all_services = all_services + self.clcs
        if len(self.walruses) > 0:
            all_services = all_services + self.walruses
            
        if len(self.osgs) > 0:
            all_services = all_services + self.osgs
            
        for partition in self.partitions.keys():

            ccs = self.partitions[partition].ccs
            if len(ccs) > 0:
                all_services = all_services + ccs
                
            scs = self.partitions[partition].scs
            if len(scs) > 0:
                all_services = all_services + scs
            
            vbs = self.partitions[partition].vbs
            if len(vbs) > 0:
                all_services = all_services + vbs
        return all_services
            
    def start_all(self):
        all_services = self.get_all_services() 
        for service in all_services:
            try:
                service.start()
            except Exception, e:
                self.tester.debug("Caught an exception when trying to turn up a service. Probably was already running")
                
    def get_conclusive_enabled_clc(self):
        self.reset()
        try:
            first_clc = self.get("eucalyptus")
        except:
            first_clc = None
            
        self.tester.swap_clc()
        
        try:
            second_clc = self.get("eucalyptus")
        except:
            second_clc = None
        
        ### Only one responded properly
        if second_clc is None:
            return first_clc 
        if first_clc is None:
            return second_clc
        
        ### Inconsistency among 2 CLCs
        if second_clc.host is first_clc.host:
                return first_clc
        else:
            raise Exception("Could not find a CLC to connect to")
        
    def sync_credentials(self):
        self.reset()



    def update(self, name=None):
        ### Get all services
        if self.last_updated:
            if (time.time() - self.last_updated) < 1:
                return
        self.reset()
        services = self.get(name)
        self.all_services = services
        for current_euservice in services:
            ### If this is system wide component add it to the base level array
            if re.search("eucalyptus", current_euservice.type) :
                self.clcs.append(current_euservice)
                continue
            elif re.search("walrus", current_euservice.type):
                self.walruses.append(current_euservice)
                continue
            elif re.search("osg", current_euservice.type):
                self.osgs.append(current_euservice)
                continue
            elif re.search("dns", current_euservice.type):
                self.dns = current_euservice
                continue
            elif re.search("arbitrator", current_euservice.type):
                self.arbitrators.append(current_euservice)
                continue
            ### If this is a partition specific component nest the service in its partition array
            if re.search("eucalyptus", current_euservice.partition) or re.search("bootstrap", current_euservice.partition):
                self.internal_components.append(current_euservice)
                continue
            else:
                append = False
                if current_euservice.partition in self.partitions:
                    my_partition = self.partitions[current_euservice.partition]
                else:
                    ### First time accessing this partition
                    my_partition = Partition(current_euservice.partition, self)
                    append = True
                if re.search("cluster", current_euservice.type):
                    my_partition.ccs.append(current_euservice)
                if re.search("storage", current_euservice.type):
                    my_partition.scs.append(current_euservice)
                if re.search("vmwarebroker", current_euservice.type):
                    my_partition.vbs.append(current_euservice)          
                if append:
                    self.partitions[current_euservice.partition] = my_partition
        enabled_clc = self.get_all_cloud_controllers(state='ENABLED')
        if enabled_clc:
            enabled_clc = enabled_clc[0]
            self.update_node_list(enabled_clc=enabled_clc)
        self.last_updated=time.time()
    
    def isReachable(self, address):
        return self.tester.ping(address)
        
    def modify_service(self, euservice, state):
        if not self.isReachable(self.tester.clc.hostname):
            self.tester.clc = self.tester.get_component_machines("clc")[1]
        modify_response = self.tester.clc.sys(self.eucaprefix + "/usr/sbin/euca-modify-service -s " + str(state)  + " " + euservice.name)
        if re.search("true",modify_response[0]):
            return True
        else:
            raise AssertionError("Response to modify service was not true: " + str(modify_response))
    
    def modify_process(self, euservice, command): 
        service_name = "eucalyptus-cloud"
        if re.search("cluster", euservice.type):
            service_name = "eucalyptus-cc"
        if euservice.type == self.node_type_string:
            service_name = "eucalyptus-nc"
        if not euservice.machine.found(self.tester.eucapath + "/etc/init.d/" + service_name + " " + command, "done"):
            self.tester.fail("Was unable to " +str(command) + " service: " + euservice.name + " on host "
                             + euservice.machine.hostname)
            raise Exception("Did not properly modify service")
    
    def stop(self, euservice):
        if re.search("cluster", euservice.type):
            self.modify_process(euservice, "cleanstop")
        else:
            self.modify_process(euservice, "stop")
        euservice.running = False
        
    def start(self, euservice):
        if euservice.type == 'cluster':
            if euservice.machine.get_eucalyptus_cc_is_running_status():
                euservice.running = True
                return
        else:
            if euservice.machine.get_eucalyptus_cloud_is_running_status():
                euservice.running = True
                return
        self.modify_process(euservice, "start")

    
    def enable(self,euservice):
        self.modify_service(euservice, "ENABLED")
    
    def disable(self,euservice):
        self.modify_service(euservice, "DISABLED")
        
    def wait_for_service(self, euservice, state = "ENABLED", states=None,attempt_both = True, timeout=600):
        interval = 20
        poll_count = timeout / interval
        while (poll_count > 0):
            matching_services = []
            try:
                matching_services = self.get(euservice.type, euservice.partition, attempt_both)
                for service in matching_services:
                    if states:
                        for state in states:
                            if re.search(state, service.state):
                                return service
                    else:
                        if re.search(state, service.state):
                            return service
            except Exception, e:
                self.tester.debug("Caught " + str(e) + " when trying to get services. Retrying in " + str(interval) + "s")
            poll_count -= 1
            self.tester.sleep(interval)
                
        if poll_count is 0:
            self.tester.fail("Service: " + euservice.name + " did not enter "  + state + " state")
            raise Exception("Service: " + euservice.name + " did not enter "  + state + " state")
        
    def all_services_operational(self):
        self.debug('all_services_operational starting...')
        all_services_to_check = self.get_all_services()
        self.print_services_list(all_services_to_check)
        while all_services_to_check:
            ha_counterpart = None
            service = all_services_to_check.pop()
            self.debug('Checking for operational state of services type:' + str(service.type))
            for serv in all_services_to_check:
                if not serv.isActiveActive() and serv.type == service.type and serv.partition == service.partition:
                    ha_counterpart = serv
                    break
            if ha_counterpart:
                all_services_to_check.remove(ha_counterpart)
                self.wait_for_service(service,"ENABLED")
                self.wait_for_service(service,"DISABLED")
            else:
                self.wait_for_service(service,"ENABLED")

    def wait_for_all_services_operational(self, timeout=600):
        '''
        Attempts to wait for a core set of eutester monitored services on the cloud and/or specified in the
        config file to transition to ENABLED. In the HA case will look for both an ENABLED and DISABLED service.
        '''
        start = time.time()
        elapsed = 0
        while elapsed < timeout:
            self.debug("wait_for_all_services_operational, elapsed: " + str(elapsed) + "/" + str(timeout))
            elapsed = int(time.time() - start)
            try:
                self.print_services_list()
                self.all_services_operational()
                self.debug('All services were detected as operational')
                return
            except Exception, e:
                tb = self.tester.get_traceback()
                elapsed = int(time.time() - start )
                error = tb + "\n Error waiting for all services operational, elapsed: " + \
                        str(elapsed) + "/" + str(timeout) + ", error:" + str(e)
                if elapsed < timeout:
                    self.debug(error)
                else:
                    raise Exception(error)
            time.sleep(15)



    def get_enabled_clc(self):
        clc = self.get_enabled(self.clcs)
        if clc is None or len(clc) == 0:
            raise Exception("Neither CLC is enabled")
        else:
            return clc.pop()
    
    def get_disabled_clc(self):
        clc = self.get_disabled(self.clcs)
        if clc is None or len(clc) == 0:
            raise Exception("Neither CLC is disabled")
        else:
            return clc.pop()
    
    def get_enabled_walrus(self):
        walrus = self.get_enabled(self.walruses)
        if walrus is None or len(walrus) == 0:
            raise Exception("Neither Walrus is enabled")
        else:
            return walrus.pop()

    def get_enabled_osg(self):
        osg = self.get_enabled(self.osgs)
        if osg is None:
            raise Exception("No Object Storage Gateways are enabled")
        else:
            return osg
    
    def get_disabled_walrus(self):
        walrus = self.get_disabled(self.walruses)
        if walrus is None or len(walrus) == 0:
            raise Exception("Neither Walrus is disabled")
        else:
            return walrus.pop()

    def get_enabled_osgs(self):
        '''Returns list of enabled osgs'''
        enabled_osgs = self.get_enabled(self.osgs)
        if enabled_osgs is None:
            raise Exception("No OSG is enabled")
        else:
            return enabled_osgs

    def get_enabled_dns(self):
        dns = self.get_enabled([self.dns])
        if dns is None or len(dns) == 0:
            raise Exception("DNS service is not available")
        else:
            return dns.pop()
    
    #modified by zhill to include *all* enabled services of this type, uses a list return
    def get_enabled(self, list_of_services):
        '''Returns a list of the services of the requested type that are in ENABLED state'''
        self.update()
        enabled_services = []
        for service in list_of_services:
            if service.isEnabled():
                enabled_services.append(service)
                
        return enabled_services;
    
    def get_disabled(self, list_of_services):
        '''Returns a list of the services of the requested type that are in DISABLED state'''
        self.update()
        disabled_services = []
        for service in list_of_services:
            if service.isDisabled():
                disabled_services.append(service)
            
        return disabled_services;










# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: matt.clark@eucalyptus.com

import sys
from boto.resultset import ResultSet
from boto.ec2.tag import Tag
from boto.ec2.ec2object import TaggedEC2Object, EC2Object
from boto.ec2.connection import EC2ResponseError


class ConversionTask(TaggedEC2Object):
    _IMPORTINSTANCE = 'importinstance'
    _IMPORTVOLUME  = 'importvolume'

    def __init__(self, connection=None):
        super(ConversionTask, self).__init__(connection)
        self.connection = connection
        self.conversiontaskid = None
        self.expirationtime = None
        self.state = None
        self.statusmessage = None
        self._importinstancetask = None
        self._importvolumetask = None
        self._instance = None
        self._snapshots = None
        self.tags = None
        self.notfound = False

    def __repr__(self):
        volumes = ",".join([vol.id for vol in self.volumes])
        return ('ConversionTask:"{0}", state:"{1}", instance:"{2}", '
                'volumes:"{3}", status:"{4}"'
                .format(str(self.conversiontaskid),
                        self.state,
                        self.instanceid,
                           volumes,
                        self.statusmessage))

    @property
    def availabilityzone(self):
        if self.importvolumes:
            return self.importvolumes[0].availabilityzone
        return None

    @property
    def importvolumes(self):
        if self._importinstancetask:
            return self._importinstancetask.importvolumes
        if self._importvolumetask:
            return [self._importvolumetask]
        return []

    @property
    def platform(self):
        if self._importinstancetask:
            return self._importinstancetask.platform
        return None

    @property
    def instanceid(self):
        if self._importinstancetask:
            return self._importinstancetask.instanceid
        return None

    @property
    def instance(self):
        if not self._instance and self.instanceid:
            ins = self._instance = self.connection.get_only_instances(
                instance_ids=[self.instanceid])
            if ins:
                self._instance = ins[0]
        return self._instance

    @property
    def volumes(self):
        '''
        Volumes are updated during the task and there may not be a volume(s)
        associated with a task response right away, EUCA-9337
        '''
        ret = []
        for im in self.importvolumes:
            if im and im.volume:
                ret.append(im.volume)
        return ret

    @property
    def snapshots(self):
        if not self._snapshots:
            self._snapshots = []
            for volume in self.volumes:
                self._snapshots.extend(self.connection.get_all_snapshots(
                    filters={'volume-id':volume.id}))
        return self._snapshots

    @property
    def image_id(self):
        if self.instance:
            return self.instance.image_id

    @property
    def tasktype(self):
        if self._importinstancetask:
            return self._IMPORTINSTANCE
        elif self._importvolumetask:
            return self._IMPORTVOLUME
        else:
            return None

    @property
    def id(self):
        return self.conversiontaskid

    @id.setter
    def id(self, taskid):
        self.conversiontaskid = taskid

    def cancel(self):
        params = {'ConversionTaskId':str(self.conversiontaskid)}
        task = self.connection.get_object('CancelConversionTask',
                                          params,
                                          ConversionTask,
                                          verb='POST')
        if task:
            self.update(updatedtask=task)
        return task

    def update(self, updatedtask=None):
        params = {}
        params['ConversionTaskId'] = str(self.conversiontaskid)
        if not updatedtask:
            updatedtask = self.connection.get_object('DescribeConversionTasks',
                                                     params,
                                                     ConversionTask,
                                                     verb='POST')
        if updatedtask:
            self.__dict__.update(updatedtask.__dict__)
        else:
            print sys.stderr, 'Update. Failed to find task:"{0}"'\
                .format(str(self.conversiontaskid))
            self.notfound = True

    def startElement(self, name, attrs, connection):
        ename = name.replace('euca:','')
        elem = super(ConversionTask, self).startElement(name,
                                                        attrs,
                                                        connection)
        if elem is not None:
            return elem
        if ename == 'importVolume':
            self._importvolumetask = ImportVolume(connection=connection)
            self.importvolumes.append(self._importvolumetask)
            return self._importvolumetask
        elif ename == 'importInstance':
            self._importinstancetask = ImportInstance(connection=connection)
            return self._importinstancetask
        elif ename == 'tagSet' or ename == 'resourceTagSet':
            self.tags = ResultSet([('item', Tag)])
            return self.tags
        else:
            return None

    def endElement(self, name, value, connection):
        ename = name.lower().replace('euca:','')
        if ename == 'conversiontaskid':
            self.conversiontaskid = value
        elif ename == 'expirationtime':
            self.expirationtime = value
        elif ename == 'state':
            self.state = value
        elif ename == 'statusmessage':
            self.statusmessage = value
        else:
            setattr(self, ename, value)



class ImportVolume(EC2Object):
    def __init__(self, connection=None):
        super(ImportVolume, self).__init__(connection)
        self.bytesconverted = None
        self.availabilityzone = None
        self.image = None
        self.volumes = self
        self._importvolume = None

    @property
    def volume_id(self):
        if self._importvolume:
            return self._importvolume.id
        else:
            return None

    @property
    def volume_size(self):
        if self._importvolume:
            return self._importvolume.size
        else:
            return None

    @property
    def volume(self):
        if self._importvolume:
            return self._importvolume.volume
        else:
            return None

    @volume.setter
    def volume(self, newvol):
        if self._importvolume:
            if newvol.id == self._importvolume.id:
                self._importvolume.volume = newvol
            else:
                raise ValueError('Can not set volume with different '
                                 'id than original')

    def startElement(self, name, attrs, connection):
        elem = super(ImportVolume, self).startElement(name,
                                                      attrs,
                                                      connection)
        self.connection = connection
        if elem is not None:
            return elem
        ename = name.lower().replace('euca:','')
        if ename == 'image':
            self.image = ConversionTaskImage(connection=connection)
            return self.image
        elif ename == 'volume':
            self._importvolume = ConversionTaskVolume(connection=connection)
            return self._importvolume
        else:
            return None

    def endElement(self, name, value, connection):
        ename = name.lower().replace('euca:','')
        if ename == 'bytesconverted':
            self.bytesconverted = value
        elif ename == 'availabilityzone':
            self.availabilityzone = value
        elif ename != 'importvolume':
            setattr(self, ename, value)


class ImportInstance(EC2Object):
    def __init__(self, connection=None):
        super(ImportInstance, self).__init__(connection)
        self.bytesconverted = None
        self.image = None
        self.platform = None
        self.instanceid = None
        self.importvolumes = []

    def startElement(self, name, attrs, connection):
        elem = super(ImportInstance, self).startElement(name,
                                                            attrs,
                                                            connection)
        if elem is not None:
            return elem
        ename = name.lower().replace('euca:','')
        if ename == 'image':
            self.image = ConversionTaskImage(connection=connection)
            return self.image
        elif ename == 'volumes':
            self.importvolumes = ResultSet([('item', ImportVolume),
                                            ('euca:item', ImportVolume)])
            return self.importvolumes
        else:
            return None

    def endElement(self, name, value, connection):
        ename = name.lower().replace('euca:','')
        if ename == 'bytesconverted':
            self.bytesconverted = value
        elif ename == 'availabilityzone':
            self.availabilityzone = value
        elif ename == 'platform':
            self.platform = value
        elif ename == 'instanceid':
            self.instanceid = value
        elif ename != 'importinstance':
            setattr(self, ename, value)

class ConversionTaskImage(object):
    def __init__(self, connection=None):
        self.connection = connection
        self.format = None
        self.importmanifesturl = None
        self.size = None

    def __repr__(self):
        return 'ConversionTaskImage:' + str(self.importmanifesturl)

    def startElement(self, name, value, connection):
        pass

    def endElement(self, name, value, connection):
        ename = name.lower().replace('euca:','')
        if ename == 'format':
            self.format = value
        elif ename == 'size':
            self.size = value
        elif ename == 'importmanifesturl':
            self.importmanifesturl = value
        elif ename != 'image':
            setattr(self, ename, value)

class ConversionTaskVolume(EC2Object):
    def __init__(self, connection=None):
        super(ConversionTaskVolume, self).__init__(connection)
        self.id = None
        self.size = None
        self._volume = None

    def __repr__(self):
        return 'ConversionTaskVolume:' + str(self.id)

    @property
    def volume(self):
        if self.connection and not self._volume:
            if (self.id):
                try:
                    volumes = self.connection.get_all_volumes([self.id])
                    for volume in volumes:
                        if volume.id == self.id:
                            self.volume = volume
                            break
                except EC2ResponseError as EE:
                    print sys.stderr, 'Failed to fetch volume:' + \
                                      str(self.id) + ' for ' \
                                      'import task:' + str(EE)
        return self._volume

    @volume.setter
    def volume(self, newvol):
        self._volume = newvol

    def startElement(self, name, attrs, connection):
        elem = super(ConversionTaskVolume, self).startElement(name,
                                                              attrs,
                                                              connection)
        if elem is not None:
            return elem
        pass

    def endElement(self, name, value, connection):
        ename = name.lower().replace('euca:','')
        if ename == 'id':
            self.id = value
        elif ename == 'size':
            self.size = value
        elif ename != 'volume':
            setattr(self, ename, value)



#!/usr/bin/python
import os
import time
import random
from eucaops import Eucaops
from eutester.eutestcase import EutesterTestCase
from testcases.cloud_user.instances.instancetest import InstanceBasics

class NCAdmin(EutesterTestCase, InstanceBasics):

    def __init__(self, config_file="cloud.conf", password="foobar"):
        self.tester = Eucaops( config_file=config_file, password=password)
        self.servman = self.tester.service_manager
        self.nc_list = self.tester.get_component_machines("nc")
        self.group = self.tester.add_group(group_name="group-" + str(time.time()))
        self.tester.authorize_group_by_name(group_name=self.group.name )
        self.tester.authorize_group_by_name(group_name=self.group.name, port=-1, protocol="icmp" )
        ### Generate a keypair for the instance
        self.keypair = self.tester.add_keypair( "keypair-" + str(time.time()))
        self.keypath = '%s/%s.pem' % (os.curdir, self.keypair.name)
        self.image = self.tester.get_emi(root_device_type="instance-store")
        zones = self.tester.ec2.get_all_zones()
        self.zone = random.choice(zones).name
        self.reservation = None
        self.conf_file = '%s/etc/eucalyptus/eucalyptus.conf' % self.tester.eucapath
        self.nc_restart_time = 30

    def restart_nc(self):
        ## Use reboot testcase to ensure we are able to attach volumes and run instances
        testcase = self.create_testcase_from_method(self.Reboot)

        ## run at least as many instances as there are NCs
        for i in xrange(len(self.nc_list)):
            testcase.run()

        for nc in self.nc_list:
            nc.sys('service eucalyptus-nc restart')

        ## Wait for ncs to come back up
        self.tester.sleep(self.nc_restart_time)

        ## rerun testcase
        for i in xrange(len(self.nc_list)):
            testcase.run()

    def replace_conf_property(self, nc, property, replacement):
        self.default_conf_property(nc, property)
        nc.sys('echo "' + property + '=' + replacement + '" >> ' + self.conf_file)

    def default_conf_property(self,nc, property):
        nc.sys('sed -i \'s/^' +  property + '/#' + property  +'/g\' ' + self.conf_file)

    def disable_caching(self):
        ## Use basic instance testcase testcase to ensure we are able to attach volumes and run instances
        testcase = self.create_testcase_from_method(self.BasicInstanceChecks)
        property = 'NC_CACHE_SIZE'
        ## run at least as many instances as there are NCs
        for i in xrange(len(self.nc_list)):
            testcase.run()

        for nc in self.nc_list:
            self.replace_conf_property(nc,  property, "0")
            command_list = ['service eucalyptus-nc stop',
                            'rm -rf {0}/var/lib/eucalyptus/instances/cache/*'.format(self.tester.eucapath),
                            'service eucalyptus-nc start']
            for command in command_list:
                nc.sys(command)

        ## Wait for ncs to come back up
        self.tester.sleep(self.nc_restart_time)

        testcase = self.create_testcase_from_method(self.Churn)
        ## rerun testcase

        testcase.run()
        for nc in self.nc_list:
            self.default_conf_property(nc, property)

    def cleanup(self):
        self.tester.cleanup_artifacts()

    def run_suite(self):
        self.testlist = []
        testlist = self.testlist
        #testlist.append(self.create_testcase_from_method(self.restart_nc))
        testlist.append(self.create_testcase_from_method(self.disable_caching))
        self.run_test_case_list(testlist)
        self.cleanup()

if __name__ == "__main__":
    parser = NCAdmin.get_parser()
    args = parser.parse_args()
    nc_admin_suite = NCAdmin(config_file=args.config, password = args.password)
    nc_admin_suite.run_suite()
   



#!/usr/bin/env python
#
#
# Description:  This script upgrades a Eucalyptus cloud
import re
from testcases.cloud_admin.install_euca import Install

class Upgrade(Install):
    def __init__(self):
        super(Upgrade, self).__init__(download_creds=True)
        self.clc_service = self.tester.service_manager.get_enabled_clc()
        self.clc = self.clc_service.machine
        self.old_version = self.clc.sys("cat /etc/eucalyptus/eucalyptus-version")[0]
        for machine in self.tester.config["machines"]:
            if re.search(machine.distro.name, "vmware"):
                self.add_enterprise_repo()
                break

    def upgrade_packages(self):
        for machine in self.tester.config["machines"]:
            if machine.distro.name is "vmware":
                continue
            if self.args.nogpg:
                machine.upgrade(nogpg=True)
            else:
                machine.upgrade("eucalyptus")
            ## IF its a CLC and we have a SAN we need to install the san package after upgrade before service start
            if re.search("^3.1", self.old_version):
                if hasattr(self.args, 'ebs_storage_manager'):
                    if re.search("SANManager" ,self.args.ebs_storage_manager):
                        if re.search("clc", " ".join(machine.components)):
                            if hasattr(self.args, 'san_provider'):
                                if re.search("EquallogicProvider", self.args.san_provider):
                                    pass # Nothing to install on CLC for this case
                                if re.search("NetappProvider", self.args.san_provider):
                                    machine.install("eucalyptus-enterprise-storage-san-netapp-libs")
                                if re.search("EmcVnxProvider", self.args.san_provider):
                                    machine.install("eucalyptus-enterprise-storage-san-emc-libs")
                        if re.search("sc", " ".join(machine.components)):
                            if hasattr(self.args, 'san_provider'):
                                if re.search("EquallogicProvider", self.args.san_provider):
                                    machine.install("eucalyptus-enterprise-storage-san-equallogic")
                                if re.search("NetappProvider", self.args.san_provider):
                                    machine.install("eucalyptus-enterprise-storage-san-netapp")
                                if re.search("EmcVnxProvider", self.args.san_provider):
                                    machine.install("eucalyptus-enterprise-storage-san-emc")
            if re.search("^3.2", self.old_version):
                if hasattr(self.args, 'ebs_storage_manager'):
                    if re.search("SANManager" ,self.args.ebs_storage_manager):
                        if re.search("clc", " ".join(machine.components)):
                            if hasattr(self.args, 'san_provider'):
                                if re.search("NetappProvider", self.args.san_provider):
                                    for zone in self.tester.get_zones():
                                        machine.sys("source " + self.tester.credpath + "/eucarc && " +
                                                    self.tester.eucapath + "/usr/sbin/euca-modify-property -p " +
                                                    zone + ".storage.chapuser=" + self.tester.id_generator())
            new_version = machine.sys("cat /etc/eucalyptus/eucalyptus-version")[0]
            if not self.args.nightly and re.match( self.old_version, new_version):
                raise Exception("Version before (" + self.old_version +") and version after (" + new_version + ") are the same")

    def UpgradeAll(self):
        self.add_euca_repo()
        try:
            self.clc.sys("rpm -qa | grep eucalyptus-enterprise", code=0)
            self.add_enterprise_repo()
        except Exception, e:
            pass
        self.stop_components()
        self.upgrade_packages()
        self.start_components()
        self.tester.service_manager.all_services_operational()

if __name__ == "__main__":
    testcase = Upgrade()
    ### Either use the list of tests passed from config/command line to determine what subset of tests to run
    list = testcase.args.tests or [ "UpgradeAll"]
    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )
        ### Run the EutesterUnitTest objects

    result = testcase.run_test_case_list(unit_list,clean_on_exit=True)
    exit(result)


#!/usr/bin/env python
# Copyright 2011-2012 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from eucaops import Eucaops
from xml.etree import ElementTree
from eutester import xmlrunner
import argparse
import unittest
import httplib
import base64

arg_credpath = None

class StsUI(unittest.TestCase):
    """
    Tests for STS UI Authentication.

    Available tests are:
      - testIssueToken - basic test for token issuance
    """
    account_name = 'test-sts-ui-account'
    admin_password = 'password'

    def setUp(self):
        self.tester = Eucaops( credpath=arg_credpath )
        self.createTestUser()

    def tearDown(self):
        self.deleteTestUser()
        self.tester = None

    def testIssueToken(self):
        """
        Test basic token issuance
        """
        ip = self.tester.tokens.region.endpoint
        port = self.tester.tokens.port
        path = self.tester.tokens.path
        api_version = self.tester.tokens.APIVersion
        url_template = '{0}?Version={1}&Action=GetSessionToken'
        sts_url = url_template.format( path, api_version )

        account = self.account_name
        username = 'admin'
        password = self.admin_password
        user_b64 = base64.b64encode( username )
        account_b64 = base64.b64encode( account )
        user_account = '{0}@{1}'.format( user_b64, account_b64 )
        creds = '{0}:{1}'.format( user_account, password )
        basic_header = "Basic {0}".format( base64.b64encode( creds.encode('utf8') ) )

        headers = {
            "Authorization": basic_header
        }
        conn = httplib.HTTPConnection( ip, port=port, timeout=30000 )
        conn.request( "GET", sts_url, headers=headers )
        response = conn.getresponse()

        self.assertEquals( 200, response.status, msg='Get token failed, bad HTTP status code: {0}'.format(response.status) )

        data = response.read()
        response.close()
        conn.close()

        rootElement = ElementTree.fromstring( data )
        resultElement = self.childElement( rootElement, 'GetSessionTokenResult' )
        credentialsElement = self.childElement( resultElement, 'Credentials')
        accessKeyIdElement = self.childElement( credentialsElement, 'AccessKeyId')
        secretAccessKeyElement = self.childElement( credentialsElement, 'SecretAccessKey')
        sessionTokenElement = self.childElement( credentialsElement, 'SessionToken')
        expirationElement = self.childElement( credentialsElement, 'Expiration')
        self.assertTrue( len( accessKeyIdElement.text ) > 0, msg='Missing accessKeyId' )
        self.assertTrue( len( secretAccessKeyElement.text ) > 0, msg='Missing secretAccessKey' )
        self.assertTrue( len( sessionTokenElement.text ) > 0, msg='Missing sessionToken' )
        self.assertTrue( len( expirationElement.text ) > 0, msg='Missing expiration' )
        self.tester.debug( "Got session token with access key: " + accessKeyIdElement.text )
        self.tester.debug( "Got session token with expiration: " + expirationElement.text )

    def childElement( self, parentElement, name ):
        childElement = parentElement.find( '{{https://sts.amazonaws.com/doc/2011-06-15/}}{0}'.format( name ) )
        self.assertTrue( childElement is not None, msg='STS response invalid, could not find {0}'.format( name ) )
        return childElement

    def createTestUser(self):
        self.tester.create_account(self.account_name)
        self.tester.euare.get_response('UpdateLoginProfile', {
            'DelegateAccount': self.account_name,
            'UserName': 'admin',
            'Password': self.admin_password
        })
        self.tester.euare.get_response('CreateAccessKey',  {
            'DelegateAccount': self.account_name,
            'UserName': 'admin',
        })

    def deleteTestUser(self):
        self.tester.delete_account(self.account_name,recursive=True)


if __name__ == "__main__":
    ## If given command line arguments, use them as test names to launch
    parser = argparse.ArgumentParser(prog='ststest.py',
        version='Test Case [ststest.py] Version 0.1',
        description='''Test token service functionality and features
                       on a Eucalyptus Cloud.''',
        usage="%(prog)s --credpath=<path to creds> [--xml] [--tests=test1,..testN]")
    parser.add_argument('--credpath',
        help="path to user credentials", default=".eucarc")
    parser.add_argument('--xml',
        help="to provide JUnit style XML output", action="store_true", default=False)
    parser.add_argument('--tests', nargs='+',
        help="test cases to be executed",
        default= ['testIssueToken'])
    args = parser.parse_args()
    arg_credpath = args.credpath
    for test in args.tests:
        if args.xml:
            file = open("test-" + test + "result.xml", "w")
            result = xmlrunner.XMLTestRunner(file).run(StsUI(test))
        else:
            result = unittest.TextTestRunner(verbosity=2).run(StsUI(test))
        if result.wasSuccessful():
            pass
        else:
            exit(1)


# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: zach@eucalyptus.com

'''
Created on Apr 24, 2012

@author: zhill
'''

import sys, argparse, string, hashlib
import boto
from boto.s3.connection import OrdinaryCallingFormat
from boto.s3.connection import S3Connection
from boto.s3.bucket import Bucket
from boto.s3.key import Key
from boto.s3.deletemarker import DeleteMarker
from boto.s3.acl import ACL
from boto.s3.acl import Policy
from boto.s3.acl import Grant
import boto.s3.prefix
from boto.exception import S3ResponseError
from boto.exception import S3CreateError
from boto.s3.connection import Location

printheader='**********************'

def print_header(message):
    print "\n" + printheader + "--" + message + "--" + printheader + "\n"

def get_config():
    parser = argparse.ArgumentParser()        
    parser.add_argument('--endpoint')
    parser.add_argument('--port', type=int, default=8773)
    parser.add_argument('--path',default='/services/Walrus')
    parser.add_argument('--access_key')
    parser.add_argument('--secret_key')
    parser.add_argument('unittest_args', nargs='*')    
    testconfig = parser.parse_args()
    
    sys.argv[1:] = testconfig.unittest_args
    return testconfig

def check_hashes(eTag=None, data=None):
        hasher = hashlib.md5()
        hasher.update(data)
        data_hash = hasher.hexdigest()
        if data_hash != eTag:
            print "Hash/eTag mismatch: \nhash = " + data_hash + "\neTag= " + eTag
            return False
        return True
    
def clean_bucket(bucket):
    """Deletes the contents of the bucket specified and the bucket itself"""
    try :
        print "Getting bucket listing for " + bucket.name        
        print "Iterating throught the bucket"
        key_list = bucket.list()        
        print "Starting loop"
        for k in key_list:
            if isinstance(k, boto.s3.prefix.Prefix):
                print "Skipping prefix"
                continue
            print "Deleting key: " + k.name
            bucket.delete_key(k)
        bucket.delete()
    except S3ResponseError as e:
        print "Exception caught doing bucket cleanup."
        if e.status == 409:
            #Do version cleanup
            print "Cleaning up versioning artifacts"
            try:
                keys = bucket.get_all_versions()
                for k in keys:
                    if isinstance(k, Key):
                        print "Got version: " + k.name + "--" + k.version_id + "-- Delete marker? " + str(k.delete_marker)
                        print "Deleting key: " + k.name
                        bucket.delete_key(key_name=k.name,version_id=k.version_id)
                    elif isinstance(k, DeleteMarker):
                        print "Got marker: " + k.name + "--" + k.version_id + "--" + str(k.is_latest)
                        print "Deleting delete marker"
                        bucket.delete_key(key_name=k.name,version_id=k.version_id)
                print "Deleting bucket " + bucket.name
                bucket.delete()
            except Exception as e:
                print "Exception deleting versioning artifacts: " + e.message

s3_groups = {
             "all_users":"http://acs.amazonaws.com/groups/global/AllUsers",
             "authenticated_users":"http://acs.amazonaws.com/groups/global/AuthenticatedUsers",
             "log_delivery":"http://acs.amazonaws.com/groups/s3/LogDelivery"
             }
        

def get_canned_acl(owner_id=None,canned_acl=None,bucket_owner_id=None):
    if owner_id == None or canned_acl == None:
        return None
    
    owner_fc_grant = Grant(permission="FULL_CONTROL",user_id=owner_id)
    built_acl = ACL()
    built_acl.add_grant(owner_fc_grant)
        
    if canned_acl == "public-read":
        built_acl.add_grant(Grant(permission="READ",uri=s3_groups["all_users"]))        
    elif canned_acl == "public-read-write":
        built_acl.add_grant(Grant(permission="READ",uri=s3_groups["all_users"]))
        built_acl.add_grant(Grant(permission="WRITE",uri=s3_groups["all_users"]))                
    elif canned_acl == "authenticated-read":
        built_acl.add_grant(Grant(permission="READ",uri=s3_groups["authenticated_users"]))        
    elif canned_acl == "log-delivery-write":
        built_acl.add_grant(Grant(permission="WRITE",uri=s3_groups["log_delivery"]))        
    elif canned_acl == "bucket-owner-read":
        built_acl.add_grant(Grant(permission="READ",user_id=bucket_owner_id))        
    elif canned_acl == "bucket-owner-full-control":
        built_acl.add_grant(Grant(permission="FULL_CONTROL",user_id=bucket_owner_id))        
    else:
        #No canned-acl value found
        return None
    return built_acl  




# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: vic.iglesias@eucalyptus.com

import time

class TaggedResource():
    def __init__(self):
        pass

    def create_tags(self, tags, timeout=600):
        self.tester.debug("Current tags: " + str(self.tags))
        self.tester.create_tags([self.id], tags)
        self.wait_for_tags(tags, timeout=timeout)

    def wait_for_tags(self, tags, creation=True, timeout=60):
        start= time.time()
        elapsed = 0
        while elapsed < timeout:
            self.update()
            applied_tags =  self.convert_tag_list_to_dict(self.tester.ec2.get_all_tags(filters={u'resource_id':self.id}))
            self.tester.debug("Current tags: " + str(applied_tags))
            found_keys = 0
            for key, value in tags.iteritems():
                if key in applied_tags:
                    found_keys += 1
                    self.tester.debug("Found key # " + str(found_keys)  + " out of " + str(len(tags))  + ":" + key)
            if creation:
                if found_keys == len(tags):
                    return True
                else:
                    pass
            else:
                if found_keys == 0:
                    return True
                else:
                    pass
            elapsed = int(time.time() - start)
            time.sleep(5)
        raise Exception("Did not apply tags within " + str(timeout) + " seconds")

    def convert_tag_list_to_dict(self, list):
        new_dict = {}
        for tag in list:
            new_dict[tag.name] = tag.value
        return new_dict

    def delete_tags(self, tags, timeout=600):
        self.tester.debug("Current tags: " + str(self.tags))
        self.tester.delete_tags([self.id], tags)
        self.wait_for_tags(tags, creation=False, timeout=timeout)


#!/usr/bin/python

from eucaops import Eucaops
from eutester.eutestcase import EutesterTestCase

class SampleTest(EutesterTestCase):
    def __init__(self):
        self.setuptestcase()
        self.setup_parser()
        self.parser.add_argument("--account-number",default=1)
        self.parser.add_argument("--account-prefix",default="test-account-")
        self.parser.add_argument("--group-prefix",default="test-group-")
        self.parser.add_argument("--user-prefix",default="test-user-")
        self.parser.add_argument("--password-prefix",default="mypassword")
        self.parser.add_argument("--user-number",default=1)
        self.get_args()
        # Setup basic eutester object
        self.tester = Eucaops(config_file=self.args.config, password=self.args.password, credpath=self.args.credpath)

    def clean_method(self):
        pass

    def CreateUsers(self):
        """
        This is where the test description goes
        """
        allow_all_policy = """{
          "Statement": [
            {
             "Action": "*",
              "Effect": "Allow",
              "Resource": "*"
            }]
            }"""

        for i in xrange(self.args.account_number):
            account_name = self.args.account_prefix + str(i)
            group_name = self.args.group_prefix + str(i)
            password = self.args.password_prefix + str(i)
            self.tester.create_account(account_name)
            self.tester.create_group(group_name, "/",account_name)
            self.tester.attach_policy_group(group_name,"allow-all", allow_all_policy, account_name)
            for k in xrange(self.args.user_number):
                user_name = self.args.user_prefix + str(k)
                self.tester.create_user(user_name, "/", account_name)
                self.tester.add_user_to_group(group_name, user_name, account_name)
                self.tester.create_login_profile(user_name, password, account_name)

if __name__ == "__main__":
    testcase = SampleTest()
    ### Use the list of tests passed from config/command line to determine what subset of tests to run
    ### or use a predefined list
    list = testcase.args.tests or ["CreateUsers"]

    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )

    ### Run the EutesterUnitTest objects
    result = testcase.run_test_case_list(unit_list,clean_on_exit=True)
    exit(result)





#!/usr/bin/env python

from __future__ import division
import time
import os
import hashlib
import tempfile
from math import ceil
from cStringIO import StringIO
from concurrent.futures import ThreadPoolExecutor
from eucaops import Eucaops
from eucaops import S3ops
from eutester.eutestcase import EutesterTestCase


class OSGConcurrency(EutesterTestCase):

    def __init__(self):
        self.setuptestcase()
        self.setup_parser()
        self.parser.add_argument("-b", "--buckets", type=int, default=5)
        self.parser.add_argument("-o", "--objects", type=int, default=5)
        self.parser.add_argument("-T", "--threads", type=int, default=5)
        self.parser.add_argument("-S", "--object-size", type=int, default=64, help="Object size in KB")
        self.parser.add_argument("-M", "--mpu-threshold", type=int, default=5120,
                                 help="Multipart upload is used when the object size is bigger than the mpu-threshold "
                                      "value in Kilobyte. Any value less than 5120KB will result single file upload. "
                                      "Default value is used when not passed as an argument.")
        self.get_args()
        # Setup basic eutester object
        if self.args.region:
            self.tester = S3ops( credpath=self.args.credpath, region=self.args.region)
        else:
            self.tester = Eucaops(credpath=self.args.credpath, config_file=self.args.config,password=self.args.password)
        self.start = time.time()

        self.bucket_names = []
        self.bucket_name = "concurrency-" + str(int(self.start)) + "-"

        for i in xrange(self.args.buckets):
            bucket_name = self.bucket_name + str(i)
            self.bucket_names.append(bucket_name)
            self.tester.create_bucket(bucket_name)

        self.temp_files = []

    def clean_method(self):
        with ThreadPoolExecutor(max_workers=self.args.threads) as executor:
            for i in xrange(self.args.buckets):
                executor.submit(self.tester.clear_bucket(self.bucket_names[i]))
        for tf in self.temp_files:
            tf.close()

    def get_object(self, bucket, key_name, meta=True):
        """
        Writes the object to a temp file and returns the meta info of the object e.g hash, name.
        Returns the downloaded object when meta is set to False.
        """
        self.debug("Getting object '" + key_name + "'")
        ret_key = bucket.get_key(key_name)
        temp_object = tempfile.NamedTemporaryFile(mode="w+b", prefix="eutester-mpu")
        self.temp_files.append(temp_object)
        ret_key.get_contents_to_file(temp_object)
        if meta:
            return {'name': temp_object.name, 'hash': self.get_hash(temp_object.name)}
        return temp_object

    def single_upload(self, bucket, key_name, file_path):
        key = bucket.new_key(key_name)
        key.set_contents_from_filename(file_path)
        self.debug("Uploaded key '" + key_name + "' to bucket '" + bucket.name + "'")
        return key

    def multipart_upload(self, bucket, key_name, eufile):
        part_size = 1024 * self.args.mpu_threshold
        eufile.seek(0, os.SEEK_END)
        eufile_size = eufile.tell()
        num_parts = int(ceil(eufile_size / part_size))

        mpu = bucket.initiate_multipart_upload(key_name)
        self.debug("Initiated MPU. Using MPU Id: " + mpu.id)

        for i in range(num_parts):
            start = part_size * i
            file_part = open(eufile.name, 'rb')
            file_part.seek(start)
            data = file_part.read(part_size)
            file_part.close()
            mpu.upload_part_from_file(StringIO(data), i+1)
            self.debug("Uploaded part " + str(i+1) + " of '" + key_name + "' to bucket '" + bucket.name + "'")
        self.debug("Completing multipart upload of '" + key_name + "' to bucket '" +
                   bucket.name + "'" + " using mpu id: " + mpu.id)
        mpu.complete_upload()
        self.debug("Completed multipart upload of '" + key_name + "' to bucket '" + bucket.name + "'")

    def put_get_check(self, bucket_name, key_name, eu_file):
        """
        PUT objects, GET objects and then verify objects with object hash
        5MB is a hard-coded limit for MPU in OSG
        """
        bucket = self.tester.get_bucket_by_name(bucket_name)
        if (os.path.getsize(eu_file.name) > (5 * 1024 * 1024)) and (self.args.mpu_threshold >= (5 * 1024)):
            self.multipart_upload(bucket, key_name, eu_file)
        else:
            self.single_upload(bucket, key_name, eu_file.name)

        ret_object_meta = self.get_object(bucket, key_name)
        local_object_hash = self.get_hash(eu_file.name)

        self.debug("Matching local and remote hashes of object: " + eu_file.name)
        self.debug("Remote object: " + ret_object_meta['hash'])
        self.debug("Local object:  " + local_object_hash)
        if ret_object_meta['hash'] != local_object_hash:
            self.debug("return_object hash: " + ret_object_meta['hash'])
            self.debug("local_object hash: " + local_object_hash)
            self.debug("Uploaded content and downloaded content are not same.")
            return False
        return True

    def get_content(self, file_path):
        with open(file_path) as file_to_check:
            data = file_to_check.read()
        return data

    def get_hash(self, file_path):
        return hashlib.md5(self.get_content(file_path)).hexdigest()

    def create_file(self, size_in_kb, file_name="eutester-object"):
        temp_file = tempfile.NamedTemporaryFile(mode='w+b', prefix=file_name)
        self.temp_files.append(temp_file)
        temp_file.write(os.urandom(1024 * size_in_kb))
        return temp_file.name

    def concurrent_upload(self):
        self.debug("Creating object of " + str(self.args.object_size) + "KB")
        eu_file = open(self.create_file(self.args.object_size))
        thread_pool = []
        with ThreadPoolExecutor(max_workers=self.args.threads) as executor:
            for i in xrange(self.args.buckets):
                for j in xrange(self.args.objects):
                    thread_pool.append(executor.submit(self.put_get_check, bucket_name=self.bucket_names[i],
                                                       key_name=eu_file.name + str(j), eu_file=eu_file))

        for tp in thread_pool:
            try:
                if not tp.result():
                    self.fail("[CRITICAL] failed upload in thread")
            except Exception as e:
                self.fail("Found exception in thread-pool: " + e.message)

if __name__ == "__main__":
    testcase = OSGConcurrency()
    ### Use the list of tests passed from config/command line to determine what subset of tests to run
    ### or use a predefined list
    list = testcase.args.tests or ["concurrent_upload"]

    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )

    ### Run the EutesterUnitTest objects
    result = testcase.run_test_case_list(unit_list)
    exit(result)

# run testcases/cloud_user/s3/osg_concurrency_test.py --config /root/input/2b_tested.lst --password foobar --buckets 1 --objects 1 --threads 1 --object-size 10240 --mpu-threshold 0
# run testcases/cloud_user/s3/osg_concurrency_test.py --config /root/input/2b_tested.lst --password foobar --buckets 2 --objects 3 --object-size 10240 --threads 3 --mpu-threshold 5120


#!/usr/bin/env python
import time
from concurrent.futures import ThreadPoolExecutor
from eucaops import Eucaops
from eucaops import S3ops
from eutester.eutestcase import EutesterTestCase

class WalrusConcurrent(EutesterTestCase):
    def __init__(self):
        self.setuptestcase()
        self.setup_parser()
        self.parser.add_argument("-n", "--number", type=int, default=100)
        self.parser.add_argument("-c", "--concurrent", type=int, default=10)
        self.parser.add_argument("-s", "--size", type=int, default=1024)
        self.get_args()
        # Setup basic eutester object
        if self.args.region:
            self.tester = S3ops( credpath=self.args.credpath, region=self.args.region)
        else:
            self.tester = Eucaops( credpath=self.args.credpath, config_file=self.args.config,password=self.args.password)
        self.start = time.time()
        self.bucket_name = "concurrency-" + str(int(self.start))
        self.tester.create_bucket(self.bucket_name)

    def clean_method(self):
        self.tester.clear_bucket(self.bucket_name)

    def Concurrent(self):
        key_payload = self.tester.id_generator(self.args.size)
        thread_count = self.args.number
        thread_pool = []
        with ThreadPoolExecutor(max_workers=thread_count) as executor:
                for i in xrange(thread_count):
                        thread_pool.append(executor.submit(self.tester.upload_object, bucket_name=self.bucket_name, key_name="test" + str(i), contents=key_payload))
        end = time.time()
        total = end - self.start
        self.tester.debug("\nExecution time: {0}\n# of Objects: {1}\nObject Size: {2}B\nConcurrency Level of {3}".format(
                            total, self.args.number, self.args.size, self.args.concurrent))
        with ThreadPoolExecutor(max_workers=thread_count) as executor:
                for object in thread_pool:
                        thread_pool.append(executor.submit(self.tester.delete_object, object))


if __name__ == "__main__":
    testcase = WalrusConcurrent()
    ### Use the list of tests passed from config/command line to determine what subset of tests to run
    ### or use a predefined list
    list = testcase.args.tests or ["Concurrent"]

    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )

    ### Run the EutesterUnitTest objects
    result = testcase.run_test_case_list(unit_list)
    exit(result)


#!/usr/bin/python
from eucaops import Eucaops
from eutester.eutestcase import EutesterTestCase

class GatherDebug(EutesterTestCase):
    basic_commands = ['df -B M',
                      'ps aux',
                      'free',
                      'uptime']
    network_commands = ['iptables -L',
                        'iptables -L -t nat',
                        'arp -a',
                        'ip addr show',
                        'ifconfig',
                        'brctl show',
                        'route',
                        'netstat -lnp']

    euca_commands = ['cat /etc/eucalyptus/eucalyptus.conf | grep -v \'#\' ',
                     'cat /etc/eucalyptus/eucalyptus-version',
                     'ps aux | grep euca']

    clc_commands = ['euca-describe-services -E -A',
                    'euca-describe-availability-zones verbose',
                    'euca-describe-instances verbose',
                    'euca-describe-volumes verbose',
                    'euca-describe-snapshots verbose',
                    'euca-describe-keypairs verbose',
                    'euca-describe-groups verbose']
    
    sc_commands = ['ls -l /dev/',
                   'losetup -a',
                   'service tgtd status'
                   'tgtadm --lld iscsi --mode target --op show',
                   'service iscsid status'
                   'iscsiadm -m session -P 3',
                   'pvdisplay',
                   'lvdisplay',
                   'vgdisplay',
                   'mpathconf',
                   'multipath -ll']
    
    nc_commands = ['service iscsid status',
                   'iscsiadm -m session -P 3',                   
                   'mpathconf',
                   'multipath -ll',
                   'virsh list',
                   'losetup -a',
                   'dmsetup status',
                   'll /var/lib/eucalyptus/instances/**/**/**']
                   

    def __init__(self):
        self.setuptestcase()
        self.setup_parser()
        self.get_args()
        # Setup basic eutester object
        self.tester = Eucaops( config_file=self.args.config_file, password=self.args.password,download_creds=True)

    def clean_method(self):
        pass

    def run_command_list(self,machine, list):
        for command in list:
            machine.sys(command)

    def debug_clc(self, **kwargs):
        clc_commands = self.basic_commands + self.network_commands + self.euca_commands + self.clc_commands
        for machine in self.tester.get_component_machines("clc"):
            for command in clc_commands:
                machine.sys("source " + self.tester.credpath + "/eucarc && " + command)

        for account in self.tester.get_all_accounts():
            account_name = next((value for key, value in account.iteritems() if 'account_name' in key), None)
            self.tester.debug( "## Account Name: " + account_name )
            for user in self.tester.get_users_from_account(delegate_account=account_name):
                user_name = next((value for key, value in user.iteritems() if 'user_name' in key), None)
                self.tester.debug( "### User Name: " + user_name )
                for policy in self.tester.get_user_policies(user_name, delegate_account=account_name):
                    self.tester.debug( "#### User " + user_name + " Policy ####" )
                    for key, value in policy.iteritems():
                        self.tester.debug( key + ": " + value )

            for group in self.tester.get_all_groups(account_name=account_name):
                group_name = next((value for key, value in group.iteritems() if 'group_name' in key), None)
                self.tester.debug( "### Group Name: " + group_name )
                for policy in self.tester.get_group_policies(group_name, delegate_account=account_name):
                    self.tester.debug( "#### Group " + group_name + " Policy ####" )
                    for key, value in policy.iteritems():
                        self.tester.debug( key + ": " + value )

    def debug_walrus(self, **kwargs):
        walrus_commands = self.basic_commands + self.network_commands + self.euca_commands
        for machine in self.tester.get_component_machines("ws"):
            self.run_command_list(machine,walrus_commands)

    def debug_cc(self, **kwargs):
        cc_commands = self.basic_commands + self.network_commands + self.euca_commands
        for machine in self.tester.get_component_machines("cc"):
            self.run_command_list(machine,cc_commands)

    def debug_sc(self, **kwargs):
        sc_commands = self.basic_commands + self.network_commands + self.euca_commands + self.sc_commands
        for machine in self.tester.get_component_machines("sc"):
            self.run_command_list(machine,sc_commands)

    def debug_nc(self, **kwargs):
        nc_commands = self.basic_commands + self.network_commands + self.euca_commands + self.nc_commands
        for machine in self.tester.get_component_machines("nc"):
            self.run_command_list(machine,nc_commands)

    def cleanup(self):
        pass

    def DebugAll(self):
        self.debug_clc()
        self.debug_walrus()
        self.debug_cc()
        self.debug_sc()
        self.debug_sc()


if __name__ == "__main__":
    testcase = GatherDebug()
    ### Use the list of tests passed from config/command line to determine what subset of tests to run
    ### or use a predefined list  "VolumeTagging", "InstanceTagging", "SnapshotTagging", "ImageTagging"
    list = testcase.args.tests or ["DebugAll"]

    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )

    ### Run the EutesterUnitTest objects
    result = testcase.run_test_case_list(unit_list,clean_on_exit=True)
    exit(result)
   
